@@ -64,6 +64,7 @@ SW_EXTERN_C_BEGIN
 static PHP_FUNCTION(swoole_version);
 static PHP_FUNCTION(swoole_cpu_num);
 static PHP_FUNCTION(swoole_strerror);
+static PHP_FUNCTION(swoole_clear_error);
 static PHP_FUNCTION(swoole_errno);
 static PHP_FUNCTION(swoole_error_log);
 static PHP_FUNCTION(swoole_get_local_ip);
@@ -105,6 +106,17 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_defer, 0, 0, 1)
     ZEND_ARG_CALLABLE_INFO(0, callback, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_socketpair, 0, 0, 3)
+    ZEND_ARG_INFO(0, domain)
+    ZEND_ARG_INFO(0, type)
+    ZEND_ARG_INFO(0, protocol)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_test_kernel_coroutine, 0, 0, 0)
+    ZEND_ARG_INFO(0, count)
+    ZEND_ARG_INFO(0, sleep_time)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_client_select, 0, 0, 3)
     ZEND_ARG_INFO(1, read_array)
     ZEND_ARG_INFO(1, write_array)
@@ -175,6 +187,8 @@ const zend_function_entry swoole_functions[] =
     /*------swoole_coroutine------*/
     PHP_FE(swoole_coroutine_create, arginfo_swoole_coroutine_create)
     PHP_FE(swoole_coroutine_defer, arginfo_swoole_coroutine_defer)
+    PHP_FE(swoole_coroutine_socketpair, arginfo_swoole_coroutine_socketpair)
+    PHP_FE(swoole_test_kernel_coroutine, arginfo_swoole_test_kernel_coroutine)
     /*------other-----*/
     PHP_FE(swoole_client_select, arginfo_swoole_client_select)
     PHP_FALIAS(swoole_select, swoole_client_select, arginfo_swoole_client_select)
@@ -183,6 +197,7 @@ const zend_function_entry swoole_functions[] =
     PHP_FE(swoole_get_local_mac, arginfo_swoole_void)
     PHP_FE(swoole_strerror, arginfo_swoole_strerror)
     PHP_FE(swoole_errno, arginfo_swoole_void)
+    PHP_FE(swoole_clear_error, arginfo_swoole_void)
     PHP_FE(swoole_error_log, arginfo_swoole_error_log)
     PHP_FE(swoole_hashcode, arginfo_swoole_hashcode)
     PHP_FE(swoole_mime_type_add, arginfo_swoole_mime_type_write)
@@ -221,6 +236,9 @@ static const zend_module_dep swoole_deps[] = {
 #endif
 #ifdef SW_SOCKETS
      ZEND_MOD_REQUIRED("sockets")
+#endif
+#ifdef SW_USE_CURL
+     ZEND_MOD_REQUIRED("curl")
 #endif
      ZEND_MOD_END
 };
@@ -373,6 +391,14 @@ void php_swoole_register_rshutdown_callback(swoole::Callback cb, void *private_d
     rshutdown_callbacks.append(cb, private_data);
 }
 
+SW_API bool php_swoole_is_enable_coroutine() {
+    if (sw_server()) {
+        return sw_server()->is_enable_coroutine();
+    } else {
+        return SWOOLE_G(enable_coroutine);
+    }
+}
+
 static void fatal_error(int code, const char *format, ...) {
     va_list args;
     zend_object *exception;
@@ -380,8 +406,7 @@ static void fatal_error(int code, const char *format, ...) {
     exception = zend_throw_exception(swoole_error_ce, swoole::std_string::vformat(format, args).c_str(), code);
     va_end(args);
     zend_exception_error(exception, E_ERROR);
-    // should never here
-    abort();
+    exit(255);
 }
 
 /* {{{ PHP_MINIT_FUNCTION
@@ -701,9 +726,6 @@ PHP_MINIT_FUNCTION(swoole) {
     }
 
     swoole_init();
-    if (!SWOOLE_G(enable_coroutine)) {
-        SwooleG.enable_coroutine = 0;
-    }
     if (strcmp("cli", sapi_module.name) == 0 || strcmp("phpdbg", sapi_module.name) == 0) {
         SWOOLE_G(cli) = 1;
     }
@@ -775,6 +797,7 @@ PHP_MINIT_FUNCTION(swoole) {
 PHP_MSHUTDOWN_FUNCTION(swoole) {
     swoole_clean();
     zend::known_strings_dtor();
+    php_swoole_runtime_mshutdown();
 
     return SUCCESS;
 }
@@ -790,16 +813,19 @@ PHP_MINFO_FUNCTION(swoole) {
     php_info_print_table_row(2, "Version", SWOOLE_VERSION);
     snprintf(buf, sizeof(buf), "%s %s", __DATE__, __TIME__);
     php_info_print_table_row(2, "Built", buf);
-    php_info_print_table_row(2, "coroutine", "enabled");
+#if defined(SW_USE_THREAD_CONTEXT)
+    php_info_print_table_row(2, "coroutine", "enabled with thread context");
+#elif defined(SW_USE_ASM_CONTEXT)
+    php_info_print_table_row(2, "coroutine", "enabled with boost asm context");
+#else
+    php_info_print_table_row(2, "coroutine", "enabled with ucontext");
+#endif
 #ifdef SW_DEBUG
     php_info_print_table_row(2, "debug", "enabled");
 #endif
 #ifdef SW_LOG_TRACE_OPEN
     php_info_print_table_row(2, "trace_log", "enabled");
 #endif
-#ifdef SW_NO_USE_ASM_CONTEXT
-    php_info_print_table_row(2, "ucontext", "enabled");
-#endif
 #ifdef HAVE_EPOLL
     php_info_print_table_row(2, "epoll", "enabled");
 #endif
@@ -833,6 +859,9 @@ PHP_MINFO_FUNCTION(swoole) {
 #else
     php_info_print_table_row(2, "openssl", "enabled");
 #endif
+#ifdef SW_SUPPORT_DTLS
+    php_info_print_table_row(2, "dtls", "enabled");
+#endif
 #endif
 #ifdef SW_USE_HTTP2
     php_info_print_table_row(2, "http2", "enabled");
@@ -840,6 +869,9 @@ PHP_MINFO_FUNCTION(swoole) {
 #ifdef SW_USE_JSON
     php_info_print_table_row(2, "json", "enabled");
 #endif
+#ifdef SW_USE_CURL
+    php_info_print_table_row(2, "curl-native", "enabled");
+#endif
 #ifdef HAVE_PCRE
     php_info_print_table_row(2, "pcre", "enabled");
 #endif
@@ -972,6 +1004,8 @@ PHP_RINIT_FUNCTION(swoole) {
     SIGG(check) = 0;
 #endif
 
+    php_swoole_runtime_rinit();
+
     SWOOLE_G(req_status) = PHP_SWOOLE_RINIT_END;
 
     return SUCCESS;
@@ -1064,6 +1098,10 @@ static PHP_FUNCTION(swoole_hashcode) {
     }
 }
 
+static PHP_FUNCTION(swoole_clear_error) {
+    swoole_set_last_error(0);
+}
+
 PHP_FUNCTION(swoole_last_error) {
     RETURN_LONG(swoole_get_last_error());
 }
