@@ -143,7 +143,10 @@ static void event_object_free(void *data) {
 static int event_readable_callback(Reactor *reactor, swEvent *event) {
     EventObject *peo = (EventObject *) event->socket->object;
 
-    if (UNEXPECTED(sw_zend_call_function_ex2(nullptr, &peo->fci_cache_read, 1, &peo->zsocket, nullptr) != SUCCESS)) {
+    zval argv[1];
+    argv[0] = peo->zsocket;
+
+    if (UNEXPECTED(!zend::function::call(&peo->fci_cache_read, 1, argv, nullptr, php_swoole_is_enable_coroutine()))) {
         php_swoole_fatal_error(E_WARNING,
                                "%s: onRead callback handler error, fd [%d] will be removed from reactor",
                                ZSTR_VAL(swoole_event_ce->name),
@@ -160,7 +163,10 @@ static int event_readable_callback(Reactor *reactor, swEvent *event) {
 static int event_writable_callback(Reactor *reactor, Event *event) {
     EventObject *peo = (EventObject *) event->socket->object;
 
-    if (UNEXPECTED(sw_zend_call_function_ex2(nullptr, &peo->fci_cache_write, 1, &peo->zsocket, nullptr) != SUCCESS)) {
+    zval argv[1];
+    argv[0] = peo->zsocket;
+
+    if (UNEXPECTED(!zend::function::call(&peo->fci_cache_write, 1, argv, nullptr, php_swoole_is_enable_coroutine()))) {
         php_swoole_fatal_error(E_WARNING,
                                "%s: onWrite callback handler error, fd [%d] will be removed from reactor",
                                ZSTR_VAL(swoole_event_ce->name),
@@ -202,7 +208,7 @@ static int event_error_callback(Reactor *reactor, swEvent *event) {
 static void event_defer_callback(void *data) {
     zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) data;
 
-    if (UNEXPECTED(sw_zend_call_function_ex2(nullptr, fci_cache, 0, nullptr, nullptr) != SUCCESS)) {
+    if (UNEXPECTED(!zend::function::call(fci_cache, 0, nullptr, nullptr, php_swoole_is_enable_coroutine()))) {
         php_swoole_error(E_WARNING, "%s::defer callback handler error", ZSTR_VAL(swoole_event_ce->name));
     }
 
@@ -212,10 +218,8 @@ static void event_defer_callback(void *data) {
 
 static void event_end_callback(void *data) {
     zend_fcall_info_cache *fci_cache = (zend_fcall_info_cache *) data;
-
-    if (UNEXPECTED(sw_zend_call_function_ex2(nullptr, (zend_fcall_info_cache *) fci_cache, 0, nullptr, nullptr) !=
-                   SUCCESS)) {
-        php_swoole_error(E_WARNING, "%s::defer callback handler error", ZSTR_VAL(swoole_event_ce->name));
+    if (UNEXPECTED(!zend::function::call(fci_cache, 0, nullptr, nullptr, php_swoole_is_enable_coroutine()))) {
+        php_swoole_error(E_WARNING, "%s::end callback handler error", ZSTR_VAL(swoole_event_ce->name));
     }
 }
 
@@ -277,7 +281,7 @@ void php_swoole_event_wait() {
         swSignalfd_setup(sw_reactor());
     }
 #endif
-    if (!sw_reactor()->if_exit()) {
+    if (!sw_reactor()->if_exit() && !sw_reactor()->bailout) {
         // Don't disable object slot reuse while running shutdown functions:
         // https://github.com/php/php-src/commit/bd6eabd6591ae5a7c9ad75dfbe7cc575fa907eac
 #if defined(EG_FLAGS_IN_SHUTDOWN) && !defined(EG_FLAGS_OBJECT_STORE_NO_REUSE)
@@ -728,7 +732,11 @@ static PHP_FUNCTION(swoole_event_wait) {
 static PHP_FUNCTION(swoole_event_rshutdown) {
     /* prevent the program from jumping out of the rshutdown */
     zend_try {
-        PHP_FN(swoole_event_wait)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
+        if (!sw_reactor()) {
+            return;
+        }
+        php_swoole_fatal_error(E_DEPRECATED, "Event::wait() in shutdown function is deprecated");
+        php_swoole_event_wait();
     }
     zend_end_try();
 }
