@@ -214,18 +214,18 @@ static void swoole_http2_onRequest(Http2Session *client, Http2Stream *stream) {
     }
 
     add_assoc_long(zserver, "request_time", time(nullptr));
-    add_assoc_double(zserver, "request_time_float", swoole_microtime());
+    add_assoc_double(zserver, "request_time_float", microtime());
     if (serv_sock) {
         add_assoc_long(zserver, "server_port", serv_sock->info.get_port());
     }
     add_assoc_long(zserver, "remote_port", conn->info.get_port());
     add_assoc_string(zserver, "remote_addr", (char *) conn->info.get_ip());
-    add_assoc_long(zserver, "master_time", (int) conn->last_recv_time);
+    add_assoc_long(zserver, "master_time", conn->last_recv_time);
     add_assoc_string(zserver, "server_protocol", (char *) "HTTP/2");
 
     zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, server_fd, SW_SERVER_CB_onRequest);
     zval args[2] = {*ctx->request.zobject, *ctx->response.zobject};
-    if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, nullptr, SwooleG.enable_coroutine))) {
+    if (UNEXPECTED(!zend::function::call(fci_cache, 2, args, nullptr, serv->is_enable_coroutine()))) {
         stream->reset(SW_HTTP2_ERROR_INTERNAL_ERROR);
         php_swoole_error(E_WARNING, "%s->onRequest[v2] handler error", ZSTR_VAL(swoole_http_server_ce->name));
     }
