@@ -28,6 +28,7 @@ extern "C" {
 }
 
 #include "swoole_base64.h"
+#include "swoole_util.h"
 
 #ifdef SW_HAVE_ZLIB
 #include <zlib.h>
@@ -38,6 +39,9 @@ extern "C" {
 #endif
 
 using swoole::coroutine::Socket;
+using swoole::Server;
+using swoole::Connection;
+using swoole::substr_len;
 using http_response = swoole::http::Response;
 using http_context = swoole::http::Context;
 
@@ -163,6 +167,7 @@ static PHP_METHOD(swoole_http_response, cookie);
 static PHP_METHOD(swoole_http_response, rawcookie);
 static PHP_METHOD(swoole_http_response, header);
 static PHP_METHOD(swoole_http_response, initHeader);
+static PHP_METHOD(swoole_http_response, isWritable);
 static PHP_METHOD(swoole_http_response, detach);
 static PHP_METHOD(swoole_http_response, create);
 /**
@@ -194,7 +199,7 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_header, 0, 0, 2)
     ZEND_ARG_INFO(0, key)
     ZEND_ARG_INFO(0, value)
-    ZEND_ARG_INFO(0, ucwords)
+    ZEND_ARG_INFO(0, format)
 ZEND_END_ARG_INFO()
 
 #ifdef SW_USE_HTTP2
@@ -249,6 +254,7 @@ ZEND_END_ARG_INFO()
 const zend_function_entry swoole_http_response_methods[] =
 {
     PHP_ME(swoole_http_response, initHeader, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http_response, isWritable, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, cookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_MALIAS(swoole_http_response, setCookie, cookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, rawcookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
@@ -365,6 +371,23 @@ static PHP_METHOD(swoole_http_response, write) {
     RETURN_BOOL(ctx->send(ctx, http_buffer->str, http_buffer->length));
 }
 
+static bool parse_header_flags(http_context *ctx, const char *key, size_t keylen, uint32_t &header_flags) {
+    if (SW_STRCASEEQ(key, keylen, "Server")) {
+        header_flags |= HTTP_HEADER_SERVER;
+    } else if (SW_STRCASEEQ(key, keylen, "Connection")) {
+        header_flags |= HTTP_HEADER_CONNECTION;
+    } else if (SW_STRCASEEQ(key, keylen, "Date")) {
+        header_flags |= HTTP_HEADER_DATE;
+    } else if (SW_STRCASEEQ(key, keylen, "Content-Length") && ctx->parser.method != PHP_HTTP_HEAD) {
+        return false;
+    } else if (SW_STRCASEEQ(key, keylen, "Content-Type")) {
+        header_flags |= HTTP_HEADER_CONTENT_TYPE;
+    } else if (SW_STRCASEEQ(key, keylen, "Transfer-Encoding")) {
+        header_flags |= HTTP_HEADER_TRANSFER_ENCODING;
+    }
+    return true;
+}
+
 static void http_build_header(http_context *ctx, swString *response, size_t body_length) {
     char *buf = sw_tg_buffer()->str;
     size_t l_buf = sw_tg_buffer()->size;
@@ -383,41 +406,50 @@ static void http_build_header(http_context *ctx, swString *response, size_t body
     }
     response->append(buf, n);
 
+    uint32_t header_flags = 0x0;
+
     /**
      * http header
      */
     zval *zheader =
         sw_zend_read_property_ex(swoole_http_response_ce, ctx->response.zobject, SW_ZSTR_KNOWN(SW_ZEND_STR_HEADER), 0);
-    uint32_t header_flag = 0x0;
     if (ZVAL_IS_ARRAY(zheader)) {
         const char *key;
         uint32_t keylen;
         int type;
         zval *zvalue;
 
+        auto add_header = [](swoole::String *response, const char *key, size_t l_key, zval *value) {
+            if (ZVAL_IS_NULL(value)) {
+                return;
+            }
+            zend::String str_value(value);
+            str_value.rtrim();
+            if (http_has_crlf(str_value.val(), str_value.len())) {
+                return;
+            }
+            response->append(key, l_key);
+            response->append(SW_STRL(": "));
+            response->append(str_value.val(), str_value.len());
+            response->append(SW_STRL("\r\n"));
+        };
+
         SW_HASHTABLE_FOREACH_START2(Z_ARRVAL_P(zheader), key, keylen, type, zvalue) {
             // TODO: numeric key name neccessary?
             if (UNEXPECTED(!key || ZVAL_IS_NULL(zvalue))) {
                 continue;
             }
-            if (SW_STRCASEEQ(key, keylen, "Server")) {
-                header_flag |= HTTP_HEADER_SERVER;
-            } else if (SW_STRCASEEQ(key, keylen, "Connection")) {
-                header_flag |= HTTP_HEADER_CONNECTION;
-            } else if (SW_STRCASEEQ(key, keylen, "Date")) {
-                header_flag |= HTTP_HEADER_DATE;
-            } else if (SW_STRCASEEQ(key, keylen, "Content-Length") && ctx->parser.method != PHP_HTTP_HEAD) {
-                continue;  // ignore
-            } else if (SW_STRCASEEQ(key, keylen, "Content-Type")) {
-                header_flag |= HTTP_HEADER_CONTENT_TYPE;
-            } else if (SW_STRCASEEQ(key, keylen, "Transfer-Encoding")) {
-                header_flag |= HTTP_HEADER_TRANSFER_ENCODING;
+            if (!parse_header_flags(ctx, key, keylen, header_flags)) {
+                continue;
             }
-            if (!ZVAL_IS_NULL(zvalue)) {
-                zend::String str_value(zvalue);
-                n = sw_snprintf(
-                    buf, l_buf, "%.*s: %.*s\r\n", (int) keylen, key, (int) str_value.len(), str_value.val());
-                response->append(buf, n);
+            if (ZVAL_IS_ARRAY(zvalue)) {
+                zval *zvalue_2;
+                SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(zvalue), zvalue_2) {
+                    add_header(response, key, keylen, zvalue_2);
+                }
+                SW_HASHTABLE_FOREACH_END();
+            } else {
+                add_header(response, key, keylen, zvalue);
             }
         }
         SW_HASHTABLE_FOREACH_END();
@@ -440,7 +472,7 @@ static void http_build_header(http_context *ctx, swString *response, size_t body
         SW_HASHTABLE_FOREACH_END();
     }
 
-    if (!(header_flag & HTTP_HEADER_SERVER)) {
+    if (!(header_flags & HTTP_HEADER_SERVER)) {
         response->append(ZEND_STRL("Server: " SW_HTTP_SERVER_SOFTWARE "\r\n"));
     }
 
@@ -451,17 +483,17 @@ static void http_build_header(http_context *ctx, swString *response, size_t body
         return;
     }
 
-    if (!(header_flag & HTTP_HEADER_CONNECTION)) {
+    if (!(header_flags & HTTP_HEADER_CONNECTION)) {
         if (ctx->keepalive) {
             response->append(ZEND_STRL("Connection: keep-alive\r\n"));
         } else {
             response->append(ZEND_STRL("Connection: close\r\n"));
         }
     }
-    if (!(header_flag & HTTP_HEADER_CONTENT_TYPE)) {
+    if (!(header_flags & HTTP_HEADER_CONTENT_TYPE)) {
         response->append(ZEND_STRL("Content-Type: text/html\r\n"));
     }
-    if (!(header_flag & HTTP_HEADER_DATE)) {
+    if (!(header_flags & HTTP_HEADER_DATE)) {
         date_str = php_swoole_format_date((char *) ZEND_STRL(SW_HTTP_DATE_FORMAT), time(nullptr), 0);
         n = sw_snprintf(buf, l_buf, "Date: %s\r\n", date_str);
         response->append(buf, n);
@@ -470,7 +502,7 @@ static void http_build_header(http_context *ctx, swString *response, size_t body
 
     if (ctx->send_chunked) {
         SW_ASSERT(body_length == 0);
-        if (!(header_flag & HTTP_HEADER_TRANSFER_ENCODING)) {
+        if (!(header_flags & HTTP_HEADER_TRANSFER_ENCODING)) {
             response->append(ZEND_STRL("Transfer-Encoding: chunked\r\n"));
         }
     }
@@ -673,6 +705,14 @@ static PHP_METHOD(swoole_http_response, initHeader) {
     RETURN_TRUE;
 }
 
+static PHP_METHOD(swoole_http_response, isWritable) {
+    http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
+    if (!ctx || (ctx->end || ctx->detached)) {
+        RETURN_FALSE;
+    }
+    RETURN_TRUE;
+}
+
 static PHP_METHOD(swoole_http_response, end) {
     http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx)) {
@@ -805,8 +845,8 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
 _skip_copy:
 #endif
     if (ctx->upgrade && !ctx->co_socket) {
-        swServer *serv = (swServer *) ctx->private_data;
-        swConnection *conn = serv->get_connection_by_session_id(ctx->fd);
+        Server *serv = (Server *) ctx->private_data;
+        Connection *conn = serv->get_connection_verify(ctx->fd);
         if (conn && conn->websocket_status == WEBSOCKET_STATUS_HANDSHAKE) {
             if (ctx->response.status == 101) {
                 conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
@@ -824,42 +864,40 @@ _skip_copy:
     RETURN_TRUE;
 }
 
-bool swoole_http_response_set_header(
-    http_context *ctx, const char *k, size_t klen, const char *v, size_t vlen, bool ucwords) {
+bool swoole_http_response_set_header(http_context *ctx, const char *k, size_t klen, const char *v, size_t vlen, bool format) {
+    zval ztmp;
+    ZVAL_STRINGL(&ztmp, v, vlen);
+    Z_ADDREF(ztmp);
+    return swoole_http_response_set_header(ctx, k, klen, &ztmp, format);
+}
+
+bool swoole_http_response_set_header(http_context *ctx, const char *k, size_t klen, zval *zvalue, bool format) {
     if (UNEXPECTED(klen > SW_HTTP_HEADER_KEY_SIZE - 1)) {
         php_swoole_error(E_WARNING, "header key is too long");
+        Z_TRY_DELREF_P(zvalue);
         return false;
     }
 
-    if (http_has_crlf(v, vlen)) {
+    if (http_has_crlf(k, klen)) {
+        Z_TRY_DELREF_P(zvalue);
         return false;
     }
 
     zval *zheader = swoole_http_init_and_read_property(
         swoole_http_response_ce, ctx->response.zobject, &ctx->response.zheader, ZEND_STRL("header"));
-    if (ucwords) {
-        char key_buf[SW_HTTP_HEADER_KEY_SIZE];
-        swoole_strlcpy(key_buf, k, sizeof(key_buf));
+    if (format) {
+        swoole_strlcpy(sw_tg_buffer()->str, k, SW_HTTP_HEADER_KEY_SIZE);
 #ifdef SW_USE_HTTP2
         if (ctx->http2) {
-            swoole_strtolower(key_buf, klen);
+            swoole_strtolower(sw_tg_buffer()->str, klen);
         } else
 #endif
         {
-            http_header_key_format(key_buf, klen);
-        }
-        if (UNEXPECTED(!v)) {
-            add_assoc_null_ex(zheader, key_buf, klen);
-        } else {
-            add_assoc_stringl_ex(zheader, key_buf, klen, (char *) v, vlen);
-        }
-    } else {
-        if (UNEXPECTED(!v)) {
-            add_assoc_null_ex(zheader, k, klen);
-        } else {
-            add_assoc_stringl_ex(zheader, k, klen, (char *) v, vlen);
+            http_header_key_format(sw_tg_buffer()->str, klen);
         }
+        k = sw_tg_buffer()->str;
     }
+    add_assoc_zval_ex(zheader, k, klen, zvalue);
     return true;
 }
 
@@ -947,7 +985,8 @@ static PHP_METHOD(swoole_http_response, sendfile) {
 }
 
 static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const bool url_encode) {
-    char *name = nullptr, *value = nullptr, *path = nullptr, *domain = nullptr, *samesite = nullptr, *priority = nullptr;
+    char *name = nullptr, *value = nullptr, *path = nullptr, *domain = nullptr, *samesite = nullptr,
+         *priority = nullptr;
     zend_long expires = 0;
     size_t name_len, value_len = 0, path_len = 0, domain_len = 0, samesite_len = 0, priority_len = 0;
     zend_bool secure = 0, httponly = 0;
@@ -970,7 +1009,7 @@ static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const
         RETURN_FALSE;
     }
 
-    int cookie_size = name_len /* + value_len */ + path_len + domain_len + 100;
+    size_t cookie_size = name_len /* + value_len */ + path_len + domain_len + 100;
     char *cookie = nullptr, *date = nullptr;
 
     if (name_len > 0 && strpbrk(name, "=,; \t\r\n\013\014") != nullptr) {
@@ -1076,23 +1115,24 @@ static PHP_METHOD(swoole_http_response, status) {
 }
 
 static PHP_METHOD(swoole_http_response, header) {
-    char *k, *v;
-    size_t klen, vlen;
-    zend_bool ucwords = 1;
+    char *k;
+    size_t klen;
+    zval *zvalue;
+    zend_bool format = 1;
 
     ZEND_PARSE_PARAMETERS_START(2, 3)
     Z_PARAM_STRING(k, klen)
-    Z_PARAM_STRING_EX(v, vlen, 1, 0)
+    Z_PARAM_ZVAL(zvalue)
     Z_PARAM_OPTIONAL
-    Z_PARAM_BOOL(ucwords)
+    Z_PARAM_BOOL(format)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx)) {
         RETURN_FALSE;
     }
-
-    RETURN_BOOL(swoole_http_response_set_header(ctx, k, klen, v, vlen, ucwords));
+    Z_TRY_ADDREF_P(zvalue);
+    RETURN_BOOL(swoole_http_response_set_header(ctx, k, klen, zvalue, format));
 }
 
 #ifdef SW_USE_HTTP2
@@ -1139,7 +1179,6 @@ static PHP_METHOD(swoole_http_response, ping) {
     SW_CHECK_RETURN(swoole_http2_server_ping(ctx));
 }
 
-
 static PHP_METHOD(swoole_http_response, goaway) {
     http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx)) {
