@@ -14,25 +14,16 @@
   +----------------------------------------------------------------------+
  */
 #include "php_swoole_cxx.h"
+#include "swoole_socket.h"
 #include "swoole_util.h"
 
 #include "thirdparty/php/standard/proc_open.h"
+#include "thirdparty/php/curl/curl_arginfo.h"
+
 #include <unordered_map>
 #include <initializer_list>
 
 /* openssl */
-#ifndef OPENSSL_NO_TLS1_METHOD
-#define HAVE_TLS1 1
-#endif
-#ifndef OPENSSL_NO_TLS1_1_METHOD
-#define HAVE_TLS11 1
-#endif
-#ifndef OPENSSL_NO_TLS1_2_METHOD
-#define HAVE_TLS12 1
-#endif
-#if OPENSSL_VERSION_NUMBER >= 0x10101000 && !defined(OPENSSL_NO_TLS1_3)
-#define HAVE_TLS13 1
-#endif
 #ifndef OPENSSL_NO_ECDH
 #define HAVE_ECDH 1
 #endif
@@ -69,6 +60,12 @@ static PHP_FUNCTION(swoole_time_sleep_until);
 static PHP_FUNCTION(swoole_stream_select);
 static PHP_FUNCTION(swoole_stream_socket_pair);
 static PHP_FUNCTION(swoole_user_func_handler);
+
+#ifdef SW_USE_CURL
+void swoole_native_curl_init(int module_number);
+void swoole_native_curl_shutdown();
+#endif
+
 SW_EXTERN_C_END
 
 static int socket_set_option(php_stream *stream, int option, int value, void *ptrparam);
@@ -113,8 +110,8 @@ struct php_swoole_netstream_data_t {
     bool blocking;
 };
 
-static bool hook_init = false;
-static int hook_flags = 0;
+static bool runtime_hook_init = false;
+static int runtime_hook_flags = 0;
 
 static struct {
     php_stream_transport_factory tcp;
@@ -132,6 +129,13 @@ static struct {
     nullptr,
 };
 
+static std::vector<std::string> unsafe_functions {
+    "pcntl_fork",
+    "pcntl_wait",
+    "pcntl_waitpid",
+    "pcntl_sigtimedwait",
+};
+
 static const zend_function_entry swoole_runtime_methods[] =
 {
     PHP_ME(swoole_runtime, enableCoroutine, arginfo_swoole_runtime_enableCoroutine, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -142,18 +146,16 @@ static const zend_function_entry swoole_runtime_methods[] =
 // clang-format on
 
 static php_stream_wrapper ori_php_plain_files_wrapper;
+static php_stream_ops ori_php_stream_stdio_ops;
 
-#if PHP_VERSION_ID < 70200
-typedef void (*zif_handler)(INTERNAL_FUNCTION_PARAMETERS);
-#endif
+static void hook_func(const char *name, size_t l_name, zif_handler handler = nullptr, zend_internal_arg_info *arg_info = nullptr);
+static void unhook_func(const char *name, size_t l_name);
+static bool hook_internal_functions(const zend_function_entry *fes);
 
 #define SW_HOOK_FUNC(f) hook_func(ZEND_STRL(#f), PHP_FN(swoole_##f))
 #define SW_UNHOOK_FUNC(f) unhook_func(ZEND_STRL(#f))
 
-static void hook_func(const char *name, size_t l_name, zif_handler handler = nullptr);
-static void unhook_func(const char *name, size_t l_name);
-
-static zend_array *function_table = nullptr;
+static zend_array *tmp_function_table = nullptr;
 
 SW_EXTERN_C_BEGIN
 #include "ext/standard/file.h"
@@ -177,28 +179,34 @@ void php_swoole_runtime_minit(int module_number) {
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_SLEEP", PHPCoroutine::HOOK_SLEEP);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_PROC", PHPCoroutine::HOOK_PROC);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_CURL", PHPCoroutine::HOOK_CURL);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_NATIVE_CURL", PHPCoroutine::HOOK_NATIVE_CURL);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_BLOCKING_FUNCTION", PHPCoroutine::HOOK_BLOCKING_FUNCTION);
+    SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_SOCKETS", PHPCoroutine::HOOK_SOCKETS);
     SW_REGISTER_LONG_CONSTANT("SWOOLE_HOOK_ALL", PHPCoroutine::HOOK_ALL);
-
+#ifdef SW_USE_CURL
+    swoole_native_curl_init(module_number);
+#endif
     swoole_proc_open_init(module_number);
 }
 
 struct real_func {
     zend_function *function;
     zif_handler ori_handler;
+    zend_internal_arg_info *ori_arg_info;
+    uint32_t ori_fn_flags;
+    uint32_t ori_num_args;
     zend_fcall_info_cache *fci_cache;
     zval name;
 };
 
+void php_swoole_runtime_rinit() {
+    tmp_function_table = (zend_array *) emalloc(sizeof(zend_array));
+    zend_hash_init(tmp_function_table, 8, nullptr, nullptr, 0);
+}
+
 void php_swoole_runtime_rshutdown() {
-    if (!hook_init) {
-        return;
-    }
-
-    hook_init = false;
-
     void *ptr;
-    ZEND_HASH_FOREACH_PTR(function_table, ptr) {
+    ZEND_HASH_FOREACH_PTR(tmp_function_table, ptr) {
         real_func *rf = reinterpret_cast<real_func *>(ptr);
         /**
          * php library function
@@ -211,9 +219,15 @@ void php_swoole_runtime_rshutdown() {
         efree(rf);
     }
     ZEND_HASH_FOREACH_END();
-    zend_hash_destroy(function_table);
-    efree(function_table);
-    function_table = nullptr;
+    zend_hash_destroy(tmp_function_table);
+    efree(tmp_function_table);
+    tmp_function_table = nullptr;
+}
+
+void php_swoole_runtime_mshutdown() {
+#ifdef SW_USE_CURL
+    swoole_native_curl_shutdown();
+#endif
 }
 
 static inline char *parse_ip_address_ex(const char *str, size_t str_len, int *portno, int get_err, zend_string **err) {
@@ -555,7 +569,7 @@ static inline int socket_accept(php_stream *stream, Socket *sock, php_stream_xpo
     Socket *clisock = sock->accept();
 
 #ifdef SW_USE_OPENSSL
-    if (clisock != nullptr && clisock->open_ssl) {
+    if (clisock != nullptr && clisock->ssl_is_enable()) {
         if (!clisock->ssl_handshake()) {
             sock->errCode = clisock->errCode;
             delete clisock;
@@ -653,13 +667,105 @@ static int socket_setup_crypto(php_stream *stream, Socket *sock, php_stream_xpor
     return 0;
 }
 
+static int socket_xport_crypto_setup(php_stream *stream) {
+    php_stream_xport_crypto_param param;
+    int ret;
+
+    memset(&param, 0, sizeof(param));
+    param.op = (decltype(param.op)) STREAM_XPORT_CRYPTO_OP_SETUP;
+    param.inputs.method = (php_stream_xport_crypt_method_t) 0;
+    param.inputs.session = NULL;
+
+    ret = php_stream_set_option(stream, PHP_STREAM_OPTION_CRYPTO_API, 0, &param);
+
+    if (ret == PHP_STREAM_OPTION_RETURN_OK) {
+        return param.outputs.returncode;
+    }
+
+    php_error_docref("streams.crypto", E_WARNING, "this stream does not support SSL/crypto");
+
+    return ret;
+}
+
+static int socket_xport_crypto_enable(php_stream *stream, int activate) {
+    php_stream_xport_crypto_param param;
+    int ret;
+
+    memset(&param, 0, sizeof(param));
+    param.op = (decltype(param.op)) STREAM_XPORT_CRYPTO_OP_ENABLE;
+    param.inputs.activate = activate;
+
+    ret = php_stream_set_option(stream, PHP_STREAM_OPTION_CRYPTO_API, 0, &param);
+
+    if (ret == PHP_STREAM_OPTION_RETURN_OK) {
+        return param.outputs.returncode;
+    }
+
+    php_error_docref("streams.crypto", E_WARNING, "this stream does not support SSL/crypto");
+
+    return ret;
+}
+
+static bool php_openssl_capture_peer_certs(php_stream *stream, Socket *sslsock) {
+    zval *val;
+
+    std::string peer_cert = sslsock->ssl_get_peer_cert();
+    if (peer_cert.empty()) {
+        return false;
+    }
+
+    zval argv[1];
+    ZVAL_STRINGL(&argv[0], peer_cert.c_str(), peer_cert.length());
+    zend::function::ReturnValue retval = zend::function::call("openssl_x509_read", 1, argv);
+    php_stream_context_set_option(PHP_STREAM_CONTEXT(stream), "ssl", "peer_certificate", &retval.value);
+    zval_dtor(&argv[0]);
+
+    if (NULL != (val = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "ssl", "capture_peer_cert_chain"))
+            && zend_is_true(val)) {
+        zval arr;
+        auto chain = sslsock->get_socket()->ssl_get_peer_cert_chain(INT_MAX);
+
+        if (!chain.empty()) {
+            array_init(&arr);
+            for (auto &cert : chain) {
+                zval argv[1];
+                ZVAL_STRINGL(&argv[0], cert.c_str(), cert.length());
+                zend::function::ReturnValue retval = zend::function::call("openssl_x509_read", 1, argv);
+                zval_add_ref(&retval.value);
+                add_next_index_zval(&arr, &retval.value);
+                zval_dtor(&argv[0]);
+            }
+        } else {
+            ZVAL_NULL(&arr);
+        }
+
+        php_stream_context_set_option(PHP_STREAM_CONTEXT(stream), "ssl", "peer_certificate_chain", &arr);
+        zval_ptr_dtor(&arr);
+    }
+
+    return true;
+}
+
 static int socket_enable_crypto(php_stream *stream, Socket *sock, php_stream_xport_crypto_param *cparam STREAMS_DC) {
-    if (cparam->inputs.activate && !sock->is_ssl_enable()) {
-        return sock->ssl_handshake() ? 0 : -1;
-    } else if (!cparam->inputs.activate && sock->is_ssl_enable()) {
+    if (cparam->inputs.activate && !sock->ssl_is_available()) {
+        sock->enable_ssl_encrypt();
+        if (!sock->ssl_check_context()) {
+            return -1;
+        }
+        if (!sock->ssl_handshake()) {
+            return -1;
+        }
+        return 0;
+    } else if (!cparam->inputs.activate && sock->ssl_is_available()) {
         return sock->ssl_shutdown() ? 0 : -1;
     }
-    return -1;
+
+    zval *val = php_stream_context_get_option(PHP_STREAM_CONTEXT(stream), "ssl", "capture_peer_cert");
+    if (val && zend_is_true(val)) {
+        return php_openssl_capture_peer_certs(stream, sock) ? 0 : -1;
+    }
+
+    return 0;
 }
 #endif
 
@@ -674,6 +780,12 @@ static inline int socket_xport_api(php_stream *stream, Socket *sock, php_stream_
     case STREAM_XPORT_OP_CONNECT:
     case STREAM_XPORT_OP_CONNECT_ASYNC:
         xparam->outputs.returncode = socket_connect(stream, sock, xparam);
+#ifdef SW_USE_OPENSSL
+        if (sock->ssl_is_enable()
+                && (socket_xport_crypto_setup(stream) < 0 || socket_xport_crypto_enable(stream, 1) < 0)) {
+            xparam->outputs.returncode = -1;
+        }
+#endif
         break;
     case STREAM_XPORT_OP_BIND: {
         if (sock->get_sock_domain() != AF_UNIX) {
@@ -798,17 +910,17 @@ static int socket_set_option(php_stream *stream, int option, int value, void *pt
 
             array_init(&tmp);
             switch (SSL_version(ssl)) {
-#ifdef HAVE_TLS13
+#ifdef TLS1_3_VERSION
             case TLS1_3_VERSION:
                 proto_str = "TLSv1.3";
                 break;
 #endif
-#ifdef HAVE_TLS12
+#ifdef TLS1_2_VERSION
             case TLS1_2_VERSION:
                 proto_str = "TLSv1.2";
                 break;
 #endif
-#ifdef HAVE_TLS11
+#ifdef TLS1_1_VERSION
             case TLS1_1_VERSION:
                 proto_str = "TLSv1.1";
                 break;
@@ -816,7 +928,7 @@ static int socket_set_option(php_stream *stream, int option, int value, void *pt
             case TLS1_VERSION:
                 proto_str = "TLSv1";
                 break;
-#ifdef HAVE_SSL3
+#ifdef SSL3_VERSION
             case SSL3_VERSION:
                 proto_str = "SSLv3";
                 break;
@@ -899,7 +1011,7 @@ static php_stream *socket_create(const char *proto,
     } else if (SW_STREQ(proto, protolen, "ssl") || SW_STREQ(proto, protolen, "tls")) {
 #ifdef SW_USE_OPENSSL
         sock = new Socket(resourcename[0] == '[' ? SW_SOCK_TCP6 : SW_SOCK_TCP);
-        sock->open_ssl = true;
+        sock->enable_ssl_encrypt();
 #else
         php_swoole_error(E_WARNING,
                          "you must configure with `--enable-openssl` to support ssl connection when compiling Swoole");
@@ -943,31 +1055,37 @@ static php_stream *socket_create(const char *proto,
     if (context && ZVAL_IS_ARRAY(&context->options)) {
 #ifdef SW_USE_OPENSSL
         zval *ztmp;
-        if (sock->open_ssl && php_swoole_array_get_value(Z_ARRVAL_P(&context->options), "ssl", ztmp) &&
+
+        if (sock->ssl_is_enable() && php_swoole_array_get_value(Z_ARRVAL_P(&context->options), "ssl", ztmp) &&
             ZVAL_IS_ARRAY(ztmp)) {
-            [](Socket *sock, HashTable *options) {
-                zval zalias, *ztmp;
-                array_init(&zalias);
-#define SSL_OPTION_ALIAS(name, alias)                                                                                  \
-    do {                                                                                                               \
-        if (php_swoole_array_get_value(options, name, ztmp)) {                                                         \
-            add_assoc_zval_ex(&zalias, ZEND_STRL(alias), ztmp);                                                        \
-        }                                                                                                              \
-    } while (0);
-                SSL_OPTION_ALIAS("peer_name", "ssl_host_name");
-                SSL_OPTION_ALIAS("verify_peer", "ssl_verify_peer");
-                SSL_OPTION_ALIAS("allow_self_signed", "ssl_allow_self_signed");
-                SSL_OPTION_ALIAS("cafile", "ssl_cafile");
-                SSL_OPTION_ALIAS("capath", "ssl_capath");
-                SSL_OPTION_ALIAS("local_cert", "ssl_cert_file");
-                SSL_OPTION_ALIAS("local_pk", "ssl_key_file");
-                SSL_OPTION_ALIAS("passphrase", "ssl_passphrase");
-                SSL_OPTION_ALIAS("verify_depth", "ssl_verify_depth");
-                SSL_OPTION_ALIAS("disable_compression", "ssl_disable_compression");
-#undef SSL_OPTION_ALIAS
-                php_swoole_socket_set_ssl(sock, &zalias);
-                zend_array_destroy(Z_ARRVAL(zalias));
-            }(sock, Z_ARRVAL_P(ztmp));
+
+            zval zalias;
+            array_init(&zalias);
+            zend_array *options = Z_ARRVAL_P(ztmp);
+
+            auto add_alias = [&zalias, options](const char *name, const char *alias) {
+                zval *ztmp;
+                if (php_swoole_array_get_value_ex(options, name, ztmp)) {
+                    add_assoc_zval_ex(&zalias, alias, strlen(alias), ztmp);
+                }
+            };
+
+            add_alias("peer_name", "ssl_host_name");
+            add_alias("verify_peer", "ssl_verify_peer");
+            add_alias("allow_self_signed", "ssl_allow_self_signed");
+            add_alias("cafile", "ssl_cafile");
+            add_alias("capath", "ssl_capath");
+            add_alias("local_cert", "ssl_cert_file");
+            add_alias("local_pk", "ssl_key_file");
+            add_alias("passphrase", "ssl_passphrase");
+            add_alias("verify_depth", "ssl_verify_depth");
+            add_alias("disable_compression", "ssl_disable_compression");
+
+            php_swoole_socket_set_ssl(sock, &zalias);
+            if (!sock->ssl_check_context()) {
+                goto _failed;
+            }
+            zend_array_destroy(Z_ARRVAL(zalias));
         }
 #endif
     }
@@ -975,8 +1093,67 @@ static php_stream *socket_create(const char *proto,
     return stream;
 }
 
+static ZEND_FUNCTION(swoole_display_disabled_function) {
+    zend_error(E_WARNING, "%s() has been disabled for security reasons", get_active_function_name());
+}
+
+static bool disable_func(const char *name, size_t l_name) {
+    real_func *rf = (real_func*) zend_hash_str_find_ptr(tmp_function_table, name, l_name);
+    if (rf) {
+        rf->function->internal_function.handler = ZEND_FN(swoole_display_disabled_function);
+        return true;
+    }
+
+    zend_function *zf = (zend_function*) zend_hash_str_find_ptr(EG(function_table), name, l_name);
+    if (zf == nullptr) {
+        return false;
+    }
+
+    rf = (real_func*) emalloc(sizeof(real_func));
+    sw_memset_zero(rf, sizeof(*rf));
+    rf->function = zf;
+    rf->ori_handler = zf->internal_function.handler;
+    rf->ori_arg_info = zf->internal_function.arg_info;
+    rf->ori_fn_flags = zf->internal_function.fn_flags;
+    rf->ori_num_args = zf->internal_function.num_args;
+
+    zf->internal_function.handler = ZEND_FN(swoole_display_disabled_function);
+    zf->internal_function.arg_info = nullptr;
+    zf->internal_function.fn_flags &= ~(ZEND_ACC_VARIADIC | ZEND_ACC_HAS_TYPE_HINTS | ZEND_ACC_HAS_RETURN_TYPE);
+    zf->internal_function.num_args = 0;
+
+    zend_hash_add_ptr(tmp_function_table, zf->common.function_name, rf);
+    return true;
+}
+
+static bool enable_func(const char *name, size_t l_name) {
+    real_func *rf = (real_func*) zend_hash_str_find_ptr(tmp_function_table, name, l_name);
+    if (!rf) {
+        return false;
+    }
+
+    rf->function->internal_function.handler = rf->ori_handler;
+    rf->function->internal_function.arg_info = rf->ori_arg_info;
+    rf->function->internal_function.fn_flags = rf->ori_fn_flags;
+    rf->function->internal_function.num_args = rf->ori_num_args;
+
+    return true;
+}
+
+void PHPCoroutine::disable_unsafe_function() {
+    for (auto &f : unsafe_functions) {
+        disable_func(f.c_str(), f.length());
+    }
+}
+
+void PHPCoroutine::enable_unsafe_function() {
+    for (auto &f : unsafe_functions) {
+        enable_func(f.c_str(), f.length());
+    }
+}
+
 bool PHPCoroutine::enable_hook(uint32_t flags) {
-    if (!hook_init) {
+    if (!runtime_hook_init) {
         HashTable *xport_hash = php_stream_xport_get_hash();
         // php_stream
         ori_factory.tcp = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("tcp"));
@@ -988,65 +1165,63 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
 
         // file
         memcpy((void *) &ori_php_plain_files_wrapper, &php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+        memcpy((void *) &ori_php_stream_stdio_ops, &php_stream_stdio_ops, sizeof(php_stream_stdio_ops));
 
-        function_table = (zend_array *) emalloc(sizeof(zend_array));
-        zend_hash_init(function_table, 8, nullptr, nullptr, 0);
-
-        hook_init = true;
+        runtime_hook_init = true;
     }
     // php_stream
     if (flags & PHPCoroutine::HOOK_TCP) {
-        if (!(hook_flags & PHPCoroutine::HOOK_TCP)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_TCP)) {
             if (php_stream_xport_register("tcp", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_TCP;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_TCP) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_TCP) {
             php_stream_xport_register("tcp", ori_factory.tcp);
         }
     }
     if (flags & PHPCoroutine::HOOK_UDP) {
-        if (!(hook_flags & PHPCoroutine::HOOK_UDP)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_UDP)) {
             if (php_stream_xport_register("udp", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_UDP;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_UDP) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_UDP) {
             php_stream_xport_register("udp", ori_factory.udp);
         }
     }
     if (flags & PHPCoroutine::HOOK_UNIX) {
-        if (!(hook_flags & PHPCoroutine::HOOK_UNIX)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_UNIX)) {
             if (php_stream_xport_register("unix", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_UNIX;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_UNIX) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_UNIX) {
             php_stream_xport_register("unix", ori_factory._unix);
         }
     }
     if (flags & PHPCoroutine::HOOK_UDG) {
-        if (!(hook_flags & PHPCoroutine::HOOK_UDG)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_UDG)) {
             if (php_stream_xport_register("udg", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_UDG;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_UDG) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_UDG) {
             php_stream_xport_register("udg", ori_factory.udg);
         }
     }
     if (flags & PHPCoroutine::HOOK_SSL) {
-        if (!(hook_flags & PHPCoroutine::HOOK_SSL)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_SSL)) {
             if (php_stream_xport_register("ssl", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_SSL;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_SSL) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_SSL) {
             if (ori_factory.ssl != nullptr) {
                 php_stream_xport_register("ssl", ori_factory.ssl);
             } else {
@@ -1055,13 +1230,13 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
         }
     }
     if (flags & PHPCoroutine::HOOK_TLS) {
-        if (!(hook_flags & PHPCoroutine::HOOK_TLS)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_TLS)) {
             if (php_stream_xport_register("tls", socket_create) != SUCCESS) {
                 flags ^= PHPCoroutine::HOOK_TLS;
             }
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_TLS) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_TLS) {
             if (ori_factory.tls != nullptr) {
                 php_stream_xport_register("tls", ori_factory.tls);
             } else {
@@ -1070,36 +1245,38 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
         }
     }
     if (flags & PHPCoroutine::HOOK_STREAM_FUNCTION) {
-        if (!(hook_flags & PHPCoroutine::HOOK_STREAM_FUNCTION)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_STREAM_FUNCTION)) {
             SW_HOOK_FUNC(stream_select);
             SW_HOOK_FUNC(stream_socket_pair);
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_STREAM_FUNCTION) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_STREAM_FUNCTION) {
             SW_UNHOOK_FUNC(stream_select);
             SW_UNHOOK_FUNC(stream_socket_pair);
         }
     }
     // file
     if (flags & PHPCoroutine::HOOK_FILE) {
-        if (!(hook_flags & PHPCoroutine::HOOK_FILE)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_FILE)) {
             memcpy((void *) &php_plain_files_wrapper, &sw_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+            memcpy((void *) &php_stream_stdio_ops, &sw_php_stream_stdio_ops, sizeof(php_stream_stdio_ops));
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_FILE) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_FILE) {
             memcpy((void *) &php_plain_files_wrapper, &ori_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+            memcpy((void *) &php_stream_stdio_ops, &ori_php_stream_stdio_ops, sizeof(php_stream_stdio_ops));
         }
     }
     // sleep
     if (flags & PHPCoroutine::HOOK_SLEEP) {
-        if (!(hook_flags & PHPCoroutine::HOOK_SLEEP)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_SLEEP)) {
             SW_HOOK_FUNC(sleep);
             SW_HOOK_FUNC(usleep);
             SW_HOOK_FUNC(time_nanosleep);
             SW_HOOK_FUNC(time_sleep_until);
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_SLEEP) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_SLEEP) {
             SW_UNHOOK_FUNC(sleep);
             SW_UNHOOK_FUNC(usleep);
             SW_UNHOOK_FUNC(time_nanosleep);
@@ -1108,14 +1285,14 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
     }
     // proc_open
     if (flags & PHPCoroutine::HOOK_PROC) {
-        if (!(hook_flags & PHPCoroutine::HOOK_PROC)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_PROC)) {
             SW_HOOK_FUNC(proc_open);
             SW_HOOK_FUNC(proc_close);
             SW_HOOK_FUNC(proc_get_status);
             SW_HOOK_FUNC(proc_terminate);
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_PROC) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_PROC) {
             SW_UNHOOK_FUNC(proc_open);
             SW_UNHOOK_FUNC(proc_close);
             SW_UNHOOK_FUNC(proc_get_status);
@@ -1124,21 +1301,122 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
     }
     // blocking function
     if (flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION) {
-        if (!(hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION)) {
             hook_func(ZEND_STRL("gethostbyname"), PHP_FN(swoole_coroutine_gethostbyname));
             hook_func(ZEND_STRL("exec"));
             hook_func(ZEND_STRL("shell_exec"));
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION) {
             SW_UNHOOK_FUNC(gethostbyname);
             SW_UNHOOK_FUNC(exec);
             SW_UNHOOK_FUNC(shell_exec);
         }
     }
+    if (flags & PHPCoroutine::HOOK_SOCKETS) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_SOCKETS)) {
+            hook_func(ZEND_STRL("socket_create"));
+            hook_func(ZEND_STRL("socket_create_listen"));
+            hook_func(ZEND_STRL("socket_create_pair"), PHP_FN(swoole_coroutine_socketpair));
+            hook_func(ZEND_STRL("socket_connect"));
+            hook_func(ZEND_STRL("socket_write"));
+            hook_func(ZEND_STRL("socket_read"));
+            hook_func(ZEND_STRL("socket_send"));
+            hook_func(ZEND_STRL("socket_recv"));
+            hook_func(ZEND_STRL("socket_sendto"));
+            hook_func(ZEND_STRL("socket_recvfrom"));
+            hook_func(ZEND_STRL("socket_bind"));
+            hook_func(ZEND_STRL("socket_listen"));
+            hook_func(ZEND_STRL("socket_accept"));
+            hook_func(ZEND_STRL("socket_getpeername"));
+            hook_func(ZEND_STRL("socket_getsockname"));
+            hook_func(ZEND_STRL("socket_getopt"));
+            hook_func(ZEND_STRL("socket_get_option"));
+            hook_func(ZEND_STRL("socket_setopt"));
+            hook_func(ZEND_STRL("socket_set_option"));
+            hook_func(ZEND_STRL("socket_set_block"));
+            hook_func(ZEND_STRL("socket_set_nonblock"));
+            hook_func(ZEND_STRL("socket_shutdown"));
+            hook_func(ZEND_STRL("socket_close"));
+            hook_func(ZEND_STRL("socket_clear_error"));
+            hook_func(ZEND_STRL("socket_last_error"));
+        }
+    } else {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION) {
+            SW_UNHOOK_FUNC(socket_create);
+            SW_UNHOOK_FUNC(socket_create_listen);
+            SW_UNHOOK_FUNC(socket_create_pair);
+            SW_UNHOOK_FUNC(socket_connect);
+            SW_UNHOOK_FUNC(socket_write);
+            SW_UNHOOK_FUNC(socket_read);
+            SW_UNHOOK_FUNC(socket_send);
+            SW_UNHOOK_FUNC(socket_recv);
+            SW_UNHOOK_FUNC(socket_sendto);
+            SW_UNHOOK_FUNC(socket_recvfrom);
+            SW_UNHOOK_FUNC(socket_bind);
+            SW_UNHOOK_FUNC(socket_listen);
+            SW_UNHOOK_FUNC(socket_accept);
+            SW_UNHOOK_FUNC(socket_getpeername);
+            SW_UNHOOK_FUNC(socket_getsockname);
+            SW_UNHOOK_FUNC(socket_getopt);
+            SW_UNHOOK_FUNC(socket_get_option);
+            SW_UNHOOK_FUNC(socket_setopt);
+            SW_UNHOOK_FUNC(socket_set_option);
+            SW_UNHOOK_FUNC(socket_set_block);
+            SW_UNHOOK_FUNC(socket_set_nonblock);
+            SW_UNHOOK_FUNC(socket_shutdown);
+            SW_UNHOOK_FUNC(socket_close);
+            SW_UNHOOK_FUNC(socket_clear_error);
+            SW_UNHOOK_FUNC(socket_last_error);
+        }
+    }
+
+#ifdef SW_USE_CURL
+    if (flags & PHPCoroutine::HOOK_NATIVE_CURL) {
+        if (flags & PHPCoroutine::HOOK_CURL) {
+            php_swoole_fatal_error(E_WARNING, "cannot enable both hooks HOOK_NATIVE_CURL and HOOK_CURL at same time");
+            flags ^= PHPCoroutine::HOOK_CURL;
+        }
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_NATIVE_CURL)) {
+#if PHP_VERSION_ID >= 80000
+            hook_internal_functions(swoole_native_curl_functions);
+#else
+            hook_func(ZEND_STRL("curl_close"), PHP_FN(swoole_native_curl_close));
+            hook_func(ZEND_STRL("curl_copy_handle"), PHP_FN(swoole_native_curl_copy_handle));
+            hook_func(ZEND_STRL("curl_errno"), PHP_FN(swoole_native_curl_errno));
+            hook_func(ZEND_STRL("curl_error"), PHP_FN(swoole_native_curl_error));
+            hook_func(ZEND_STRL("curl_exec"), PHP_FN(swoole_native_curl_exec));
+            hook_func(ZEND_STRL("curl_getinfo"), PHP_FN(swoole_native_curl_getinfo));
+            hook_func(ZEND_STRL("curl_init"), PHP_FN(swoole_native_curl_init));
+            hook_func(ZEND_STRL("curl_setopt"), PHP_FN(swoole_native_curl_setopt));
+            hook_func(ZEND_STRL("curl_setopt_array"), PHP_FN(swoole_native_curl_setopt_array));
+            hook_func(ZEND_STRL("curl_reset"), PHP_FN(swoole_native_curl_reset));
+            hook_func(ZEND_STRL("curl_pause"), PHP_FN(swoole_native_curl_pause));
+            hook_func(ZEND_STRL("curl_escape"), PHP_FN(swoole_native_curl_escape));
+            hook_func(ZEND_STRL("curl_unescape"), PHP_FN(swoole_native_curl_unescape));
+            #endif
+        }
+    } else {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_NATIVE_CURL) {
+            SW_UNHOOK_FUNC(curl_close);
+            SW_UNHOOK_FUNC(curl_copy_handle);
+            SW_UNHOOK_FUNC(curl_errno);
+            SW_UNHOOK_FUNC(curl_error);
+            SW_UNHOOK_FUNC(curl_exec);
+            SW_UNHOOK_FUNC(curl_getinfo);
+            SW_UNHOOK_FUNC(curl_init);
+            SW_UNHOOK_FUNC(curl_setopt);
+            SW_UNHOOK_FUNC(curl_setopt_array);
+            SW_UNHOOK_FUNC(curl_reset);
+            SW_UNHOOK_FUNC(curl_pause);
+            SW_UNHOOK_FUNC(curl_escape);
+            SW_UNHOOK_FUNC(curl_unescape);
+        }
+    }
+#endif
 
     if (flags & PHPCoroutine::HOOK_CURL) {
-        if (!(hook_flags & PHPCoroutine::HOOK_CURL)) {
+        if (!(runtime_hook_flags & PHPCoroutine::HOOK_CURL)) {
             hook_func(ZEND_STRL("curl_init"));
             hook_func(ZEND_STRL("curl_setopt"));
             hook_func(ZEND_STRL("curl_setopt_array"));
@@ -1151,7 +1429,7 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
             hook_func(ZEND_STRL("curl_multi_getcontent"));
         }
     } else {
-        if (hook_flags & PHPCoroutine::HOOK_CURL) {
+        if (runtime_hook_flags & PHPCoroutine::HOOK_CURL) {
             SW_UNHOOK_FUNC(curl_init);
             SW_UNHOOK_FUNC(curl_setopt);
             SW_UNHOOK_FUNC(curl_setopt_array);
@@ -1165,7 +1443,7 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
         }
     }
 
-    hook_flags = flags;
+    runtime_hook_flags = flags;
     return true;
 }
 
@@ -1211,7 +1489,11 @@ static PHP_METHOD(swoole_runtime, enableCoroutine) {
 }
 
 static PHP_METHOD(swoole_runtime, getHookFlags) {
-    RETURN_LONG(hook_flags);
+    if (runtime_hook_init) {
+        RETURN_LONG(runtime_hook_flags);
+    } else {
+        RETURN_LONG(PHPCoroutine::get_hook_flags());
+    }
 }
 
 static PHP_METHOD(swoole_runtime, setHookFlags) {
@@ -1516,8 +1798,13 @@ static PHP_FUNCTION(swoole_stream_select) {
     RETURN_LONG(retval);
 }
 
-static void hook_func(const char *name, size_t l_name, zif_handler handler) {
-    real_func *rf = (real_func *) zend_hash_str_find_ptr(function_table, name, l_name);
+bool hook_internal_functions(const zend_function_entry *fes) {
+    zend_unregister_functions(fes, -1, CG(function_table));
+    return zend_register_functions(NULL, fes, NULL, MODULE_PERSISTENT) == SUCCESS;
+}
+
+static void hook_func(const char *name, size_t l_name, zif_handler handler, zend_internal_arg_info *arg_info) {
+    real_func *rf = (real_func *) zend_hash_str_find_ptr(tmp_function_table, name, l_name);
     bool use_php_func = false;
     /**
      * use php library function
@@ -1559,11 +1846,11 @@ static void hook_func(const char *name, size_t l_name, zif_handler handler) {
         rf->fci_cache = func_cache;
     }
 
-    zend_hash_add_ptr(function_table, zf->common.function_name, rf);
+    zend_hash_add_ptr(tmp_function_table, zf->common.function_name, rf);
 }
 
 static void unhook_func(const char *name, size_t l_name) {
-    real_func *rf = (real_func *) zend_hash_str_find_ptr(function_table, name, l_name);
+    real_func *rf = (real_func *) zend_hash_str_find_ptr(tmp_function_table, name, l_name);
     if (rf == nullptr) {
         return;
     }
@@ -1595,6 +1882,10 @@ php_stream *php_swoole_create_stream_from_socket(php_socket_t _fd, int domain, i
     return stream;
 }
 
+php_stream_ops *php_swoole_get_ori_php_stream_stdio_ops() {
+    return &ori_php_stream_stdio_ops;
+}
+
 static PHP_FUNCTION(swoole_stream_socket_pair) {
     zend_long domain, type, protocol;
     php_stream *s1, *s2;
@@ -1641,6 +1932,6 @@ static PHP_FUNCTION(swoole_user_func_handler) {
     fci.no_separation = 1;
 #endif
 
-    real_func *rf = (real_func *) zend_hash_find_ptr(function_table, execute_data->func->common.function_name);
+    real_func *rf = (real_func *) zend_hash_find_ptr(tmp_function_table, execute_data->func->common.function_name);
     zend_call_function(&fci, rf->fci_cache);
 }
