@@ -333,10 +333,6 @@ struct ListenPort {
      * open tcp keepalive
      */
     bool open_tcp_keepalive = false;
-    /**
-     * open tcp keepalive
-     */
-    bool open_ssl_encrypt = false;
     /**
      * Sec-WebSocket-Protocol
      */
@@ -348,11 +344,13 @@ struct ListenPort {
     int kernel_socket_send_buffer_size = 0;
 
 #ifdef SW_USE_OPENSSL
-    SSL_CTX *ssl_context = nullptr;
-    swSSL_config ssl_config = {};
-    swSSL_option ssl_option = {};
+    SSLContext *ssl_context = nullptr;
+    std::unordered_map<std::string, std::shared_ptr<SSLContext>> sni_contexts;
 #ifdef SW_SUPPORT_DTLS
     std::unordered_map<int, dtls::Session *> *dtls_sessions = nullptr;
+    bool is_dtls() {
+        return ssl_context && (ssl_context->protocols & SW_SSL_DTLS);
+    }
 #endif
 #endif
 
@@ -392,7 +390,18 @@ struct ListenPort {
     void close();
     bool import(int sock);
 #ifdef SW_USE_OPENSSL
-    int enable_ssl_encrypt();
+    bool ssl_create_context(SSLContext *context);
+    bool ssl_create(Connection *conn, network::Socket *sock);
+    bool ssl_add_sni_cert(const std::string &name, SSLContext *context);
+    bool ssl_init();
+
+    void ssl_set_key_file(const std::string &file) {
+        ssl_context->key_file = file;
+    }
+
+    void ssl_set_cert_file(const std::string &file) {
+        ssl_context->cert_file = file;
+    }
 #endif
     void clear_protocol();
     inline network::Socket *get_socket() {
@@ -442,7 +451,7 @@ class Factory {
     virtual bool dispatch(SendData *) = 0;
     virtual bool finish(SendData *) = 0;
     virtual bool notify(DataHead *) = 0;
-    virtual bool end(SessionId sesion_id) = 0;
+    virtual bool end(SessionId sesion_id, int flags) = 0;
 };
 
 class BaseFactory : public Factory {
@@ -454,7 +463,7 @@ class BaseFactory : public Factory {
     bool dispatch(SendData *) override;
     bool finish(SendData *) override;
     bool notify(DataHead *) override;
-    bool end(SessionId sesion_id) override;
+    bool end(SessionId sesion_id, int flags) override;
 };
 
 class ProcessFactory : public Factory {
@@ -470,7 +479,7 @@ class ProcessFactory : public Factory {
     bool dispatch(SendData *) override;
     bool finish(SendData *) override;
     bool notify(DataHead *) override;
-    bool end(SessionId sesion_id) override;
+    bool end(SessionId sesion_id, int flags) override;
 };
 
 enum ServerEventType {
@@ -529,6 +538,11 @@ class Server {
         HOOK_PROCESS_TIMER,
     };
 
+    enum CloseFlag {
+        CLOSE_RESET = 1u << 1,
+        CLOSE_ACTIVELY = 1u << 2,
+    };
+
     /**
      * reactor thread/process num
      */
@@ -645,10 +659,14 @@ class Server {
      * asynchronous reloading
      */
     bool reload_async = true;
+    /**
+     * use event object
+     */
+    bool event_object = false;
     /**
      * use task object
      */
-    bool task_use_object = false;
+    bool task_object = false;
     /**
      * enable coroutine in task worker
      */
@@ -816,7 +834,7 @@ class Server {
      */
     std::function<void(Server *)> onManagerStart;
     std::function<void(Server *)> onManagerStop;
-    std::function<void(Server *, int, pid_t, int, int)> onWorkerError;
+    std::function<void(Server *, int, pid_t, int)> onWorkerError;
     std::function<void(Server *)> onBeforeReload;
     std::function<void(Server *)> onAfterReload;
     /**
@@ -964,6 +982,16 @@ class Server {
         return mode_ == MODE_BASE;
     }
 
+    inline bool is_enable_coroutine() {
+        if (is_task_worker()) {
+            return task_enable_coroutine;
+        } else if (is_manager()) {
+            return false;
+        } else {
+            return enable_coroutine;
+        }
+    }
+
     inline bool is_hash_dispatch_mode() {
         return dispatch_mode == SW_DISPATCH_FDMOD || dispatch_mode == SW_DISPATCH_IPMOD;
     }
@@ -1108,7 +1136,6 @@ class Server {
         Connection *conn = get_connection_verify_no_ssl(session_id);
 #ifdef SW_USE_OPENSSL
         if (conn && conn->ssl && !conn->ssl_ready) {
-            swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SSL_NOT_READY, "SSL not ready");
             return nullptr;
         }
 #endif
@@ -1160,9 +1187,10 @@ class Server {
     static int dispatch_task(Protocol *proto, network::Socket *_socket, const char *data, uint32_t length);
 
     int send_to_connection(SendData *);
-    ssize_t send_to_worker_from_master(Worker *worker, const void *data, size_t len);
+    ssize_t send_to_worker_from_master(Worker *worker, const iovec *iov, size_t iovcnt);
     ssize_t send_to_worker_from_worker(Worker *dst_worker, const void *buf, size_t len, int flags);
-    ssize_t send_to_reactor_thread(EventData *ev_data, size_t sendn, SessionId session_id);
+    ssize_t send_to_reactor_thread(const EventData *ev_data, size_t sendn, SessionId session_id);
+    ssize_t send_to_reactor_thread(const DataHead *head, const iovec *iov, size_t iovcnt, SessionId session_id);
     int reply_task_result(const char *data, size_t data_len, int flags, EventData *current_task);
 
     bool send(SessionId session_id, const void *data, uint32_t length);
@@ -1186,6 +1214,13 @@ class Server {
         return max_connection;
     }
 
+    void set_start_session_id(SessionId value) {
+        if (value > UINT_MAX) {
+            value = UINT_MAX;
+        }
+        gs->session_round = value;
+    }
+
     int create_pipe_buffers();
     void create_worker(Worker *worker);
     void destroy_worker(Worker *worker);
@@ -1260,8 +1295,6 @@ typedef swoole::ListenPort swListenPort;
 typedef swoole::Connection swConnection;
 typedef swoole::RecvData swRecvData;
 
-#define SW_MAX_SESSION_ID 0x1000000
-
 extern swoole::Server *g_server_instance;
 
 static inline swoole::Server *sw_server() {
