@@ -191,6 +191,25 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode) {
     return ret;
 }
 
+int swoole_coroutine_socket_create(int fd) {
+    if (sw_unlikely(is_no_coro())) {
+        return -1;
+    }
+    Socket *socket = new Socket(fd, SW_SOCK_RAW);
+    int _fd = socket->get_fd();
+    if (sw_unlikely(_fd < 0)) {
+        delete socket;
+    } else {
+        std::unique_lock<std::mutex> _lock(socket_map_lock);
+        socket_map[fd] = socket;
+    }
+    return 0;
+}
+
+uint8_t swoole_coroutine_socket_exists(int fd) {
+    return socket_map.find(fd) != socket_map.end();
+}
+
 ssize_t swoole_coroutine_read(int sockfd, void *buf, size_t count) {
     if (sw_unlikely(is_no_coro())) {
         return read(sockfd, buf, count);
