@@ -15,11 +15,16 @@
  */
 
 #include "swoole.h"
-#include "server.h"
-#include "websocket.h"
+#include "swoole_server.h"
+#include "swoole_websocket.h"
 
-static inline uint16_t swWebSocket_get_ext_flags(uchar opcode, uchar flags)
-{
+using swoole::Server;
+using swoole::Connection;
+using swoole::String;
+using swoole::Protocol;
+using swoole::network::Socket;
+
+static inline uint16_t swWebSocket_get_ext_flags(uchar opcode, uchar flags) {
     uint16_t ext_flags = opcode;
     ext_flags = ext_flags << 8;
     ext_flags += flags;
@@ -48,49 +53,41 @@ static inline uint16_t swWebSocket_get_ext_flags(uchar opcode, uchar flags)
  +---------------------------------------------------------------+
  */
 
-ssize_t swWebSocket_get_package_length(swProtocol *protocol, swSocket *conn, const char *buf, uint32_t length)
-{
-    //need more data
-    if (length < SW_WEBSOCKET_HEADER_LEN)
-    {
+ssize_t swWebSocket_get_package_length(Protocol *protocol, Socket *conn, const char *buf, uint32_t length) {
+    // need more data
+    if (length < SW_WEBSOCKET_HEADER_LEN) {
         return 0;
     }
 
     char mask = (buf[1] >> 7) & 0x1;
-    //0-125
+    // 0-125
     uint64_t payload_length = buf[1] & 0x7f;
     size_t header_length = SW_WEBSOCKET_HEADER_LEN;
     buf += SW_WEBSOCKET_HEADER_LEN;
 
-    //uint16_t, 2byte
-    if (payload_length == SW_WEBSOCKET_EXT16_LENGTH)
-    {
+    // uint16_t, 2byte
+    if (payload_length == SW_WEBSOCKET_EXT16_LENGTH) {
         header_length += sizeof(uint16_t);
-        if (length < header_length)
-        {
+        if (length < header_length) {
             protocol->real_header_length = header_length;
             return 0;
         }
         payload_length = ntohs(*((uint16_t *) buf));
         buf += sizeof(uint16_t);
     }
-    //uint64_t, 8byte
-    else if (payload_length == SW_WEBSOCKET_EXT64_LENGTH)
-    {
+    // uint64_t, 8byte
+    else if (payload_length == SW_WEBSOCKET_EXT64_LENGTH) {
         header_length += sizeof(uint64_t);
-        if (length < header_length)
-        {
+        if (length < header_length) {
             protocol->real_header_length = header_length;
             return 0;
         }
         payload_length = swoole_ntoh64(*((uint64_t *) buf));
         buf += sizeof(uint64_t);
     }
-    if (mask)
-    {
+    if (mask) {
         header_length += SW_WEBSOCKET_MASK_LEN;
-        if (length < header_length)
-        {
+        if (length < header_length) {
             protocol->real_header_length = header_length;
             return 0;
         }
@@ -99,25 +96,21 @@ ssize_t swWebSocket_get_package_length(swProtocol *protocol, swSocket *conn, con
     return header_length + payload_length;
 }
 
-static sw_inline void swWebSocket_mask(char *data, size_t len, const char *mask_key)
-{
+static sw_inline void swWebSocket_mask(char *data, size_t len, const char *mask_key) {
     size_t n = len / 8;
-    uint64_t mask_key64 = ((uint64_t) (*((uint32_t *) mask_key)) << 32) | *((uint32_t *) mask_key);
+    uint64_t mask_key64 = ((uint64_t)(*((uint32_t *) mask_key)) << 32) | *((uint32_t *) mask_key);
     size_t i;
 
-    for (i = 0; i < n; i++)
-    {
+    for (i = 0; i < n; i++) {
         ((uint64_t *) data)[i] ^= mask_key64;
     }
 
-    for (i = n * 8; i < len; i++)
-    {
+    for (i = n * 8; i < len; i++) {
         data[i] ^= mask_key[i % SW_WEBSOCKET_MASK_LEN];
     }
 }
 
-void swWebSocket_encode(swString *buffer, const char *data, size_t length, char opcode, uint8_t _flags)
-{
+bool swWebSocket_encode(String *buffer, const char *data, size_t length, char opcode, uint8_t _flags) {
     int pos = 0;
     char frame_header[16];
     swWebSocket_frame_header *header = (swWebSocket_frame_header *) frame_header;
@@ -129,89 +122,85 @@ void swWebSocket_encode(swString *buffer, const char *data, size_t length, char
     header->MASK = !!(_flags & SW_WEBSOCKET_FLAG_MASK);
     pos = 2;
 
-    if (length < 126)
-    {
+    if (length < 126) {
         header->LENGTH = length;
-    }
-    else if (length < 65536)
-    {
+    } else if (length < 65536) {
         header->LENGTH = SW_WEBSOCKET_EXT16_LENGTH;
         uint16_t *length_ptr = (uint16_t *) (frame_header + pos);
         *length_ptr = htons(length);
         pos += sizeof(*length_ptr);
-    }
-    else
-    {
+    } else {
         header->LENGTH = SW_WEBSOCKET_EXT64_LENGTH;
         uint64_t *length_ptr = (uint64_t *) (frame_header + pos);
         *length_ptr = swoole_hton64(length);
         pos += sizeof(*length_ptr);
     }
-    swString_append_ptr(buffer, frame_header, pos);
+    buffer->append(frame_header, pos);
     /**
      * frame body
      */
-    if (header->MASK)
-    {
-        swString_append_ptr(buffer, SW_WEBSOCKET_MASK_DATA, SW_WEBSOCKET_MASK_LEN);
-        if (length > 0)
-        {
+    if (header->MASK) {
+        buffer->append(SW_WEBSOCKET_MASK_DATA, SW_WEBSOCKET_MASK_LEN);
+        if (_flags & SW_WEBSOCKET_FLAG_ENCODE_HEADER_ONLY) {
+            return false;
+        }
+        if (length > 0) {
             size_t offset = buffer->length;
             // Warn: buffer may be extended, string pointer will change
-            swString_append_ptr(buffer, data, length);
+            buffer->append(data, length);
             swWebSocket_mask(buffer->str + offset, length, SW_WEBSOCKET_MASK_DATA);
         }
-    }
-    else
-    {
-        if (length > 0)
-        {
-            swString_append_ptr(buffer, data, length);
+    } else {
+        if (length > 0 and !(_flags & SW_WEBSOCKET_FLAG_ENCODE_HEADER_ONLY)) {
+            buffer->append(data, length);
         }
     }
+
+    return true;
 }
 
-void swWebSocket_decode(swWebSocket_frame *frame, swString *data)
-{
-    memcpy(frame, data->str, SW_WEBSOCKET_HEADER_LEN);
+bool swWebSocket_decode(swWebSocket_frame *frame, char *data, size_t length) {
+    memcpy(frame, data, SW_WEBSOCKET_HEADER_LEN);
 
-    //0-125
+    // 0-125
     size_t payload_length = frame->header.LENGTH;
     uint8_t header_length = SW_WEBSOCKET_HEADER_LEN;
-    char *buf = data->str + SW_WEBSOCKET_HEADER_LEN;
+    char *buf = data + SW_WEBSOCKET_HEADER_LEN;
 
-    //uint16_t, 2byte
-    if (frame->header.LENGTH == 0x7e)
-    {
+    // uint16_t, 2byte
+    if (frame->header.LENGTH == 0x7e) {
         payload_length = ntohs(*((uint16_t *) buf));
         header_length += 2;
     }
-    //uint64_t, 8byte
-    else if (frame->header.LENGTH > 0x7e)
-    {
+    // uint64_t, 8byte
+    else if (frame->header.LENGTH > 0x7e) {
         payload_length = swoole_ntoh64(*((uint64_t *) buf));
         header_length += 8;
     }
 
-    if (frame->header.MASK)
-    {
-        memcpy(frame->mask_key, data->str + header_length, SW_WEBSOCKET_MASK_LEN);
-        header_length += SW_WEBSOCKET_MASK_LEN;
-        if (payload_length > 0)
-        {
-            swWebSocket_mask(data->str + header_length, payload_length, frame->mask_key);
-        }
+    if (payload_length == 0) {
+        frame->header_length = header_length;
+        frame->payload_length = 0;
+        frame->payload = nullptr;
+
+        return true;
     }
 
+    if (frame->header.MASK) {
+        memcpy(frame->mask_key, data + header_length, SW_WEBSOCKET_MASK_LEN);
+        header_length += SW_WEBSOCKET_MASK_LEN;
+        swWebSocket_mask(data + header_length, payload_length, frame->mask_key);
+    }
+
+    frame->payload = data + header_length;
     frame->header_length = header_length;
-    frame->payload = data->str + header_length;
     frame->payload_length = payload_length;
+
+    return true;
 }
 
-int swWebSocket_pack_close_frame(swString *buffer, int code, char* reason, size_t length, uint8_t flags)
-{
-    if (sw_unlikely(length > SW_WEBSOCKET_CLOSE_REASON_MAX_LEN))
-    {
+int swWebSocket_pack_close_frame(String *buffer, int code, char *reason, size_t length, uint8_t flags) {
+    if (sw_unlikely(length > SW_WEBSOCKET_CLOSE_REASON_MAX_LEN)) {
         swWarn("the max length of close reason is %d", SW_WEBSOCKET_CLOSE_REASON_MAX_LEN);
         return SW_ERR;
     }
@@ -219,8 +208,7 @@ int swWebSocket_pack_close_frame(swString *buffer, int code, char* reason, size_
     char payload[SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_CLOSE_CODE_LEN + SW_WEBSOCKET_CLOSE_REASON_MAX_LEN];
     payload[0] = (char) ((code >> 8 & 0xFF));
     payload[1] = (char) ((code & 0xFF));
-    if (length > 0)
-    {
+    if (length > 0) {
         memcpy(payload + SW_WEBSOCKET_CLOSE_CODE_LEN, reason, length);
     }
     flags |= SW_WEBSOCKET_FLAG_FIN;
@@ -228,143 +216,124 @@ int swWebSocket_pack_close_frame(swString *buffer, int code, char* reason, size_
     return SW_OK;
 }
 
-void swWebSocket_print_frame(swWebSocket_frame *frame)
-{
-    printf("FIN: %x, RSV1: %d, RSV2: %d, RSV3: %d, opcode: %d, MASK: %d, length: %ld\n", frame->header.FIN,
-            frame->header.RSV1, frame->header.RSV2, frame->header.RSV3, frame->header.OPCODE, frame->header.MASK,
-            frame->payload_length);
+void swWebSocket_print_frame(swWebSocket_frame *frame) {
+    printf("FIN: %x, RSV1: %d, RSV2: %d, RSV3: %d, opcode: %d, MASK: %d, length: %ld\n",
+           frame->header.FIN,
+           frame->header.RSV1,
+           frame->header.RSV2,
+           frame->header.RSV3,
+           frame->header.OPCODE,
+           frame->header.MASK,
+           frame->payload_length);
 
-    if (frame->payload_length)
-    {
+    if (frame->payload_length) {
         printf("payload: %.*s\n", (int) frame->payload_length, frame->payload);
     }
 }
 
-int swWebSocket_dispatch_frame(swProtocol *proto, swSocket *_socket, const char *data, uint32_t length)
-{
-    swServer *serv = (swServer *) proto->private_data_2;
-    swConnection *conn = (swConnection *) _socket->object;
-    swString frame;
-    bzero(&frame, sizeof(frame));
-    frame.str = const_cast<char*>(data);
-    frame.length = length;
+int swWebSocket_dispatch_frame(Protocol *proto, Socket *_socket, const char *data, uint32_t length) {
+    Server *serv = (Server *) proto->private_data_2;
+    Connection *conn = (Connection *) _socket->object;
 
-    swString send_frame = {};
+    String send_frame = {};
     char buf[SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_CLOSE_CODE_LEN + SW_WEBSOCKET_CLOSE_REASON_MAX_LEN];
     send_frame.str = buf;
     send_frame.size = sizeof(buf);
 
     swWebSocket_frame ws;
-    swWebSocket_decode(&ws, &frame);
+    swWebSocket_decode(&ws, const_cast<char *>(data), length);
 
-    swString *frame_buffer;
+    String *frame_buffer;
     int frame_length;
     swListenPort *port;
 
     size_t offset;
-    switch (ws.header.OPCODE)
-    {
+    switch (ws.header.OPCODE) {
     case WEBSOCKET_OPCODE_CONTINUATION:
         frame_buffer = conn->websocket_buffer;
-        if (frame_buffer == NULL)
-        {
-            swWarn("bad frame[opcode=0]. remote_addr=%s:%d", swSocket_get_ip(conn->socket_type, &conn->info),
-                    swSocket_get_port(conn->socket_type, &conn->info));
+        if (frame_buffer == nullptr) {
+            swWarn("bad frame[opcode=0]. remote_addr=%s:%d", conn->info.get_ip(), conn->info.get_port());
             return SW_ERR;
         }
         offset = length - ws.payload_length;
         frame_length = length - offset;
-        port = swServer_get_port(serv, conn->fd);
-        //frame data overflow
-        if (frame_buffer->length + frame_length > port->protocol.package_max_length)
-        {
-            swWarn("websocket frame is too big, remote_addr=%s:%d", swSocket_get_ip(conn->socket_type, &conn->info),
-                    swSocket_get_port(conn->socket_type, &conn->info));
+        port = serv->get_port_by_fd(conn->fd);
+        // frame data overflow
+        if (frame_buffer->length + frame_length > port->protocol.package_max_length) {
+            swWarn("websocket frame is too big, remote_addr=%s:%d", conn->info.get_ip(), conn->info.get_port());
             return SW_ERR;
         }
-        //merge incomplete data
-        swString_append_ptr(frame_buffer, data + offset, frame_length);
-        //frame is finished, do dispatch
-        if (ws.header.FIN)
-        {
+        // merge incomplete data
+        frame_buffer->append(data + offset, frame_length);
+        // frame is finished, do dispatch
+        if (ws.header.FIN) {
             proto->ext_flags = conn->websocket_buffer->offset;
-            swReactorThread_dispatch(proto, _socket, frame_buffer->str, frame_buffer->length);
-            swString_free(frame_buffer);
-            conn->websocket_buffer = NULL;
+            proto->ext_flags |= SW_WEBSOCKET_FLAG_FIN;
+            Server::dispatch_task(proto, _socket, frame_buffer->str, frame_buffer->length);
+            delete frame_buffer;
+            conn->websocket_buffer = nullptr;
         }
         break;
 
     case WEBSOCKET_OPCODE_TEXT:
-    case WEBSOCKET_OPCODE_BINARY:
-    {
+    case WEBSOCKET_OPCODE_BINARY: {
         offset = length - ws.payload_length;
         proto->ext_flags = swWebSocket_get_ext_flags(ws.header.OPCODE, swWebSocket_get_flags(&ws));
 
-        if (!ws.header.FIN)
-        {
-            if (conn->websocket_buffer)
-            {
+        if (!ws.header.FIN) {
+            if (conn->websocket_buffer) {
                 swWarn("merging incomplete frame, bad request. remote_addr=%s:%d",
-                        swSocket_get_ip(conn->socket_type, &conn->info),
-                        swSocket_get_port(conn->socket_type, &conn->info));
+                       conn->info.get_ip(),
+                       conn->info.get_port());
                 return SW_ERR;
             }
-            conn->websocket_buffer = swString_dup(data + offset, length - offset);
+            conn->websocket_buffer = new swoole::String(data + offset, length - offset);
             conn->websocket_buffer->offset = proto->ext_flags;
-        }
-        else
-        {
-            swReactorThread_dispatch(proto, _socket, data + offset, length - offset);
+        } else {
+            Server::dispatch_task(proto, _socket, data + offset, length - offset);
         }
         break;
     }
     case WEBSOCKET_OPCODE_PING:
-        if (length >= (sizeof(buf) - SW_WEBSOCKET_HEADER_LEN))
-        {
-            swWarn("ping frame application data is too big. remote_addr=%s:%d",
-                    swSocket_get_ip(conn->socket_type, &conn->info),
-                    swSocket_get_port(conn->socket_type, &conn->info));
-            return SW_ERR;
-        }
-        else if (length == SW_WEBSOCKET_HEADER_LEN)
-        {
-            swWebSocket_encode(&send_frame, NULL, 0, WEBSOCKET_OPCODE_PONG, SW_WEBSOCKET_FLAG_FIN);
-        }
-        else
-        {
-            offset = ws.header.MASK ? SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_MASK_LEN : SW_WEBSOCKET_HEADER_LEN;
-            swWebSocket_encode(&send_frame, data += offset, length - offset, WEBSOCKET_OPCODE_PONG, SW_WEBSOCKET_FLAG_FIN);
-        }
-        swSocket_send(_socket, send_frame.str, send_frame.length, 0);
-        break;
-
     case WEBSOCKET_OPCODE_PONG:
+        if (length >= (sizeof(buf) - SW_WEBSOCKET_HEADER_LEN)) {
+            swWarn("%s frame application data is too big. remote_addr=%s:%d",
+                   ws.header.OPCODE == WEBSOCKET_OPCODE_PING ? "ping" : "pong",
+                   conn->info.get_ip(),
+                   conn->info.get_port());
+            return SW_ERR;
+        } else if (length == SW_WEBSOCKET_HEADER_LEN) {
+            data = nullptr;
+            length = 0;
+        } else {
+            offset = ws.header.MASK ? SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_MASK_LEN : SW_WEBSOCKET_HEADER_LEN;
+            data += offset;
+            length -= offset;
+        }
+        proto->ext_flags = swWebSocket_get_ext_flags(ws.header.OPCODE, swWebSocket_get_flags(&ws));
+        Server::dispatch_task(proto, _socket, data, length);
         break;
 
     case WEBSOCKET_OPCODE_CLOSE:
-        if ((length - SW_WEBSOCKET_HEADER_LEN) > SW_WEBSOCKET_CLOSE_REASON_MAX_LEN)
-        {
+        if ((length - SW_WEBSOCKET_HEADER_LEN) > SW_WEBSOCKET_CLOSE_REASON_MAX_LEN) {
             return SW_ERR;
         }
 
-        if (conn->websocket_status != WEBSOCKET_STATUS_CLOSING)
-        {
+        if (conn->websocket_status != WEBSOCKET_STATUS_CLOSING) {
             // Dispatch the frame with the same format of message frame
             offset = length - ws.payload_length;
             proto->ext_flags = swWebSocket_get_ext_flags(ws.header.OPCODE, swWebSocket_get_flags(&ws));
 
-            swReactorThread_dispatch(proto, _socket, data + offset, length - offset);
+            Server::dispatch_task(proto, _socket, data + offset, length - offset);
 
             // Client attempt to close
-            send_frame.str[0] = 0x88; // FIN | OPCODE: WEBSOCKET_OPCODE_CLOSE
+            send_frame.str[0] = 0x88;  // FIN | OPCODE: WEBSOCKET_OPCODE_CLOSE
             send_frame.str[1] = ws.payload_length;
             // Get payload and return it as it is
-            memcpy(send_frame.str + SW_WEBSOCKET_HEADER_LEN, frame.str + frame.length - ws.payload_length, ws.payload_length);
+            memcpy(send_frame.str + SW_WEBSOCKET_HEADER_LEN, data + length - ws.payload_length, ws.payload_length);
             send_frame.length = SW_WEBSOCKET_HEADER_LEN + ws.payload_length;
-            swSocket_send(_socket, send_frame.str, send_frame.length, 0);
-        }
-        else
-        {
+            _socket->send(send_frame.str, send_frame.length, 0);
+        } else {
             // Server attempt to close, frame sent by swoole_websocket_server->disconnect()
             conn->websocket_status = 0;
         }
