@@ -208,7 +208,7 @@ void Server::check_worker_exit_status(int worker_id, pid_t pid, int status) {
                WTERMSIG(status),
                WTERMSIG(status) == SIGSEGV ? "\n" SWOOLE_BUG_REPORT : "");
         if (onWorkerError != nullptr) {
-            onWorkerError(this, worker_id, pid, WEXITSTATUS(status), WTERMSIG(status));
+            onWorkerError(this, worker_id, pid, status);
         }
     }
 }
@@ -219,7 +219,6 @@ int Manager::start(Server *_server) {
 
     SwooleG.use_signalfd = 0;
     SwooleTG.reactor = nullptr;
-    SwooleG.enable_coroutine = 0;
 
     reload_workers = (Worker *) sw_calloc(_server->worker_num + _server->task_worker_num, sizeof(Worker));
     if (reload_workers == nullptr) {
