@@ -148,11 +148,9 @@ int Server::accept_connection(Reactor *reactor, Event *event) {
 
 #ifdef SW_USE_OPENSSL
         if (listen_host->ssl) {
-            if (sock->ssl_create(listen_host->ssl_context, SW_SSL_SERVER) < 0) {
+            if (!listen_host->ssl_create(conn, sock)) {
                 reactor->close(reactor, sock);
                 return SW_OK;
-            } else {
-                conn->ssl = 1;
             }
         } else {
             sock->ssl = nullptr;
@@ -164,11 +162,11 @@ int Server::accept_connection(Reactor *reactor, Event *event) {
                 return SW_OK;
             }
         } else {
-            DataHead ev = {};
+            DataHead ev{};
             ev.type = SW_SERVER_EVENT_INCOMING;
-            ev.fd = sock->fd;
-            Socket *_pipe_sock = serv->get_reactor_thread_pipe(conn->session_id, conn->reactor_id);
-            if (reactor->write(reactor, _pipe_sock, &ev, sizeof(ev)) < 0) {
+            ev.fd = conn->session_id;
+            ev.reactor_id = conn->reactor_id;
+            if (serv->send_to_reactor_thread((EventData*) &ev, sizeof(ev), conn->session_id) < 0) {
                 reactor->close(reactor, sock);
                 return SW_OK;
             }
@@ -909,7 +907,7 @@ bool Server::feedback(Connection *conn, enum ServerEventType event) {
     _send.info.reactor_id = conn->reactor_id;
 
     if (is_process_mode()) {
-        return send_to_reactor_thread((EventData *) &_send.info, sizeof(_send.info), conn->session_id) > 0;
+        return send_to_reactor_thread((EventData*) &_send.info, sizeof(_send.info), conn->session_id) > 0;
     } else {
         return send_to_connection(&_send) == SW_OK;
     }
@@ -934,88 +932,79 @@ void Server::store_pipe_fd(UnixSocket *p) {
  * @process Worker
  */
 bool Server::send(SessionId session_id, const void *data, uint32_t length) {
-    SendData _send;
-    sw_memset_zero(&_send.info, sizeof(_send.info));
-
-    if (sw_unlikely(is_master())) {
-        swoole_error_log(
-            SW_LOG_ERROR, SW_ERROR_SERVER_SEND_IN_MASTER, "can't send data to the connections in master process");
-        return false;
-    }
-
+    SendData _send{};
     _send.info.fd = session_id;
     _send.info.type = SW_SERVER_EVENT_RECV_DATA;
-    _send.data = (char *) data;
+    _send.data = (char*) data;
     _send.info.len = length;
     return factory->finish(&_send);
 }
 
 int Server::schedule_worker(int fd, SendData *data) {
-       uint32_t key = 0;
+    uint32_t key = 0;
 
-       if (dispatch_func) {
-           int id = dispatch_func(this, get_connection(fd), data);
-           if (id != SW_DISPATCH_RESULT_USERFUNC_FALLBACK) {
-               return id;
-           }
-       }
+    if (dispatch_func) {
+        int id = dispatch_func(this, get_connection(fd), data);
+        if (id != SW_DISPATCH_RESULT_USERFUNC_FALLBACK) {
+            return id;
+        }
+    }
 
-       // polling mode
-       if (dispatch_mode == SW_DISPATCH_ROUND) {
-           key = sw_atomic_fetch_add(&worker_round_id, 1);
-       }
-       // Using the FD touch access to hash
-       else if (dispatch_mode == SW_DISPATCH_FDMOD) {
-           key = fd;
-       }
-       // Using the IP touch access to hash
-       else if (dispatch_mode == SW_DISPATCH_IPMOD) {
-           Connection *conn = get_connection(fd);
-           // UDP
-           if (conn == nullptr) {
-               key = fd;
-           }
-           // IPv4
-           else if (conn->socket_type == SW_SOCK_TCP) {
-               key = conn->info.addr.inet_v4.sin_addr.s_addr;
-           }
-           // IPv6
-           else {
+    // polling mode
+    if (dispatch_mode == SW_DISPATCH_ROUND) {
+        key = sw_atomic_fetch_add(&worker_round_id, 1);
+    }
+    // Using the FD touch access to hash
+    else if (dispatch_mode == SW_DISPATCH_FDMOD) {
+        key = fd;
+    }
+    // Using the IP touch access to hash
+    else if (dispatch_mode == SW_DISPATCH_IPMOD) {
+        Connection *conn = get_connection(fd);
+        // UDP
+        if (conn == nullptr) {
+            key = fd;
+        }
+        // IPv4
+        else if (conn->socket_type == SW_SOCK_TCP) {
+            key = conn->info.addr.inet_v4.sin_addr.s_addr;
+        }
+        // IPv6
+        else {
 #ifdef HAVE_KQUEUE
-               key = *(((uint32_t *) &conn->info.addr.inet_v6.sin6_addr) + 3);
+            key = *(((uint32_t*) &conn->info.addr.inet_v6.sin6_addr) + 3);
 #elif defined(_WIN32)
                key = conn->info.addr.inet_v6.sin6_addr.u.Word[3];
 #else
                key = conn->info.addr.inet_v6.sin6_addr.s6_addr32[3];
 #endif
-           }
-       } else if (dispatch_mode == SW_DISPATCH_UIDMOD) {
-           Connection *conn = get_connection(fd);
-           if (conn == nullptr || conn->uid == 0) {
-               key = fd;
-           } else {
-               key = conn->uid;
-           }
-       }
-       // Preemptive distribution
-       else {
-           uint32_t i;
-           bool found = false;
-           for (i = 0; i < worker_num + 1; i++) {
-               key = sw_atomic_fetch_add(&worker_round_id, 1) % worker_num;
-               if (workers[key].status == SW_WORKER_IDLE) {
-                   found = true;
-                   break;
-               }
-           }
-           if (sw_unlikely(!found)) {
-               scheduler_warning = true;
-           }
-           swTraceLog(SW_TRACE_SERVER, "schedule=%d, round=%d", key, worker_round_id);
-           return key;
-       }
-       return key % worker_num;
-   }
+        }
+    } else if (dispatch_mode == SW_DISPATCH_UIDMOD) {
+        Connection *conn = get_connection(fd);
+        if (conn == nullptr || conn->uid == 0) {
+            key = fd;
+        } else {
+            key = conn->uid;
+        }
+    }
+    // Preemptive distribution
+    else {
+        uint32_t i;
+        bool found = false;
+        for (i = 0; i < worker_num + 1; i++) {
+            key = sw_atomic_fetch_add(&worker_round_id, 1) % worker_num;
+            if (workers[key].status == SW_WORKER_IDLE) {
+                found = true;
+                break;
+            }
+        }
+        if (sw_unlikely(!found)) {
+            scheduler_warning = true;
+        } swTraceLog(SW_TRACE_SERVER, "schedule=%d, round=%d", key, worker_round_id);
+        return key;
+    }
+    return key % worker_num;
+}
 
 /**
  * [Master] send to client or append to out_buffer
@@ -1122,7 +1111,7 @@ int Server::send_to_connection(SendData *_send) {
         _direct_send:
             n = _socket->send(_send_data, _send_length, 0);
             if (n == _send_length) {
-                conn->last_send_time = swoole_microtime();
+                conn->last_send_time = microtime();
                 return SW_OK;
             } else if (n > 0) {
                 _send_data += n;
@@ -1219,7 +1208,7 @@ bool Server::notify(Connection *conn, enum ServerEventType event) {
  * @process Worker
  */
 bool Server::sendfile(SessionId session_id, const char *file, uint32_t l_file, off_t offset, size_t length) {
-    if (sw_unlikely(session_id <= 0 || session_id > SW_MAX_SESSION_ID)) {
+    if (sw_unlikely(session_id <= 0)) {
         swoole_error_log(SW_LOG_WARNING, SW_ERROR_SESSION_INVALID_ID, "invalid fd[%ld]", session_id);
         return false;
     }
@@ -1345,45 +1334,7 @@ void Server::call_hook(HookType type, void *arg) {
  * [Worker]
  */
 bool Server::close(SessionId session_id, bool reset) {
-    if (sw_unlikely(is_master())) {
-        swoole_error_log(
-            SW_LOG_ERROR, SW_ERROR_SERVER_SEND_IN_MASTER, "cannot close session#%ld in master process", session_id);
-        return false;
-    }
-    Connection *conn = get_connection_verify_no_ssl(session_id);
-    if (!conn) {
-        return false;
-    }
-    // Reset send buffer, Immediately close the connection.
-    if (reset) {
-        conn->close_reset = 1;
-    }
-    // server is initiative to close the connection
-    conn->close_actively = 1;
-    swTraceLog(SW_TRACE_CLOSE, "session_id=%ld, fd=%d", session_id, conn->fd);
-
-    Worker *worker;
-    DataHead ev = {};
-
-    if (is_hash_dispatch_mode()) {
-        int worker_id = schedule_worker(conn->fd, nullptr);
-        if (worker_id != (int) SwooleG.process_id) {
-            worker = get_worker(worker_id);
-            goto _notify;
-        } else {
-            goto _close;
-        }
-    } else if (!is_worker()) {
-        worker = get_worker(conn->fd % worker_num);
-    _notify:
-        ev.type = SW_SERVER_EVENT_CLOSE;
-        ev.fd = session_id;
-        ev.reactor_id = conn->reactor_id;
-        return send_to_worker_from_worker(worker, &ev, sizeof(ev), SW_PIPE_MASTER) > 0;
-    } else {
-    _close:
-        return factory->end(session_id);
-    }
+    return factory->end(session_id, reset ? CLOSE_ACTIVELY | CLOSE_RESET: CLOSE_ACTIVELY);
 }
 
 void Server::init_signal_handler() {
@@ -1545,16 +1496,17 @@ ListenPort *Server::add_port(enum swSocket_type type, const char *host, int port
         type = (enum swSocket_type)(type & (~SW_SOCK_SSL));
         ls->type = type;
         ls->ssl = 1;
-        ls->ssl_config.prefer_server_ciphers = 1;
-        ls->ssl_config.session_tickets = 0;
-        ls->ssl_config.stapling = 1;
-        ls->ssl_config.stapling_verify = 1;
-        ls->ssl_config.ciphers = sw_strdup(SW_SSL_CIPHER_LIST);
-        ls->ssl_config.ecdh_curve = sw_strdup(SW_SSL_ECDH_CURVE);
+        ls->ssl_context = new SSLContext();
+        ls->ssl_context->prefer_server_ciphers = 1;
+        ls->ssl_context->session_tickets = 0;
+        ls->ssl_context->stapling = 1;
+        ls->ssl_context->stapling_verify = 1;
+        ls->ssl_context->ciphers = sw_strdup(SW_SSL_CIPHER_LIST);
+        ls->ssl_context->ecdh_curve = sw_strdup(SW_SSL_ECDH_CURVE);
 
         if (ls->is_dgram()) {
 #ifdef SW_SUPPORT_DTLS
-            ls->ssl_option.protocols = SW_SSL_DTLS;
+            ls->ssl_context->protocols = SW_SSL_DTLS;
             ls->dtls_sessions = new std::unordered_map<int, dtls::Session *>;
 
 #else
@@ -1571,7 +1523,7 @@ ListenPort *Server::add_port(enum swSocket_type type, const char *host, int port
         return nullptr;
     }
 #if defined(SW_SUPPORT_DTLS) && defined(HAVE_KQUEUE)
-    if (ls->ssl_option.protocols & SW_SSL_DTLS) {
+    if (ls->is_dtls()) {
         ls->socket->set_reuse_port();
     }
 #endif
@@ -1717,7 +1669,7 @@ Connection *Server::add_connection(ListenPort *ls, Socket *_socket, int server_f
     connection->fd = fd;
     connection->reactor_id = is_base_mode() ? SwooleG.process_id : fd % reactor_num;
     connection->server_fd = (sw_atomic_t) server_fd;
-    connection->last_recv_time = connection->connect_time = swoole_microtime();
+    connection->last_recv_time = connection->connect_time = microtime();
     connection->active = 1;
     connection->socket_type = ls->type;
     connection->socket = _socket;
@@ -1733,12 +1685,8 @@ Connection *Server::add_connection(ListenPort *ls, Socket *_socket, int server_f
     sw_spinlock(&gs->spinlock);
     SessionId session_id = gs->session_round;
     // get session id
-    for (uint32_t i = 0; i < max_connection; i++) {
-        session_id++;
-        if (sw_unlikely(session_id == SW_MAX_SESSION_ID)) {
-            session_id = 1;
-        }
-        Session *session = get_session(session_id);
+    SW_LOOP_N(max_connection) {
+        Session *session = get_session(++session_id);
         // available slot
         if (session->fd == 0) {
             session->fd = fd;
