@@ -224,6 +224,9 @@ static int ReactorProcess_onPipeRead(Reactor *reactor, Event *event) {
             output_buffer->clear();
         }
         break;
+    case SW_SERVER_EVENT_CLOSE:
+        factory->end(task.info.fd, Server::CLOSE_ACTIVELY);
+        break;
     default:
         break;
     }
@@ -431,7 +434,7 @@ static void ReactorProcess_onTimeout(Timer *timer, TimerNode *tnode) {
     Reactor *reactor = (Reactor *) tnode->data;
     Server *serv = (Server *) reactor->ptr;
     Event notify_ev{};
-    double now = swoole_microtime();
+    double now = microtime();
 
     if (now < serv->heartbeat_check_lasttime + 10) {
         return;
