@@ -55,13 +55,13 @@ static inline enum swReturn_code ReactorThread_verify_ssl_state(Reactor *reactor
 
     Connection *conn = (Connection *) _socket->object;
     conn->ssl_ready = 1;
-    if (port->ssl_option.client_cert_file) {
+    if (!port->ssl_context->client_cert_file.empty()) {
         if (!_socket->ssl_get_peer_certificate(sw_tg_buffer())) {
-            if (port->ssl_option.verify_peer) {
+            if (port->ssl_context->verify_peer) {
                 return SW_ERROR;
             }
         } else {
-            if (!port->ssl_option.verify_peer || _socket->ssl_verify(port->ssl_option.allow_self_signed)) {
+            if (!port->ssl_context->verify_peer || _socket->ssl_verify(port->ssl_context->allow_self_signed)) {
                 SendData task;
                 task.info.fd = _socket->fd;
                 task.info.type = SW_SERVER_EVENT_CONNECT;
@@ -132,7 +132,7 @@ static int ReactorThread_onPacketReceived(Reactor *reactor, Event *event) {
     task.info.server_fd = fd;
     task.info.reactor_id = SwooleTG.id;
     task.info.type = SW_SERVER_EVENT_RECV_DGRAM;
-    task.info.time = swoole_microtime();
+    task.info.time = microtime();
 
     pkt->socket_addr.type = pkt->socket_type = server_sock->socket_type;
 
@@ -151,9 +151,8 @@ _do_recvfrom:
 #ifdef SW_SUPPORT_DTLS
     ListenPort *port = (ListenPort *) server_sock->object;
 
-    if (port->ssl_option.protocols & SW_SSL_DTLS) {
+    if (port->is_dtls()) {
         dtls::Session *session = serv->accept_dtls_connection(port, &pkt->socket_addr);
-
         if (!session) {
             return SW_ERR;
         }
@@ -171,13 +170,11 @@ _do_recvfrom:
                 return SW_OK;
             }
         } else {
-            DataHead ev = {};
+            DataHead ev{};
             ev.type = SW_SERVER_EVENT_INCOMING;
-            ev.fd = session->socket->fd;
-            Socket *_pipe_sock = serv->get_reactor_thread_pipe(conn->session_id, conn->reactor_id);
-            ReactorThread *thread = serv->get_thread(SwooleTG.id);
-            Socket *socket = &thread->pipe_sockets[_pipe_sock->fd];
-            if (reactor->write(reactor, socket, &ev, sizeof(ev)) < 0) {
+            ev.fd = conn->session_id;
+            ev.reactor_id = conn->reactor_id;
+            if (serv->send_to_reactor_thread((EventData*) &ev, sizeof(ev), conn->session_id) < 0) {
                 reactor->close(reactor, session->socket);
                 return SW_OK;
             }
@@ -402,8 +399,7 @@ static int ReactorThread_onPipeRead(Reactor *reactor, Event *ev) {
                  * connection incoming
                  */
                 if (resp->info.type == SW_SERVER_EVENT_INCOMING) {
-                    int fd = resp->info.fd;
-                    Connection *conn = serv->get_connection(fd);
+                    Connection *conn = serv->get_connection_by_session_id(resp->info.fd);
                     if (serv->connection_incoming(reactor, conn) < 0) {
                         return reactor->close(reactor, conn->socket);
                     }
@@ -451,13 +447,13 @@ static int ReactorThread_onPipeRead(Reactor *reactor, Event *ev) {
     return SW_OK;
 }
 
-ssize_t Server::send_to_worker_from_master(Worker *worker, const void *data, size_t len) {
+ssize_t Server::send_to_worker_from_master(Worker *worker, const iovec *iov, size_t iovcnt) {
     if (SwooleTG.reactor) {
         ReactorThread *thread = get_thread(SwooleTG.id);
         Socket *socket = &thread->pipe_sockets[worker->pipe_master->fd];
-        return swoole_event_write(socket, data, len);
+        return swoole_event_writev(socket, iov, iovcnt);
     } else {
-        return worker->pipe_master->send_blocking(data, len);
+        return worker->pipe_master->writev_blocking(iov, iovcnt);
     }
 }
 
@@ -479,7 +475,7 @@ static int ReactorThread_onPipeWrite(Reactor *reactor, Event *ev) {
             // send_data->info.fd is session_id
             Connection *conn = serv->get_connection_verify(send_data->info.fd);
             if (conn) {
-                conn->last_send_time = swoole_microtime();
+                conn->last_send_time = microtime();
                 if (conn->closed) {
                     swoole_error_log(SW_LOG_NOTICE,
                                      SW_ERROR_SESSION_CLOSED_BY_SERVER,
@@ -536,7 +532,7 @@ void Server::init_reactor(Reactor *reactor) {
     for (auto port : ports) {
         if (port->is_dgram()
 #ifdef SW_SUPPORT_DTLS
-            && !(port->ssl_option.protocols & SW_SSL_DTLS)
+            && !(port->is_dtls())
 #endif
         ) {
             continue;
@@ -558,7 +554,7 @@ static int ReactorThread_onRead(Reactor *reactor, Event *event) {
     ListenPort *port = serv->get_port_by_fd(event->fd);
 #ifdef SW_USE_OPENSSL
 #ifdef SW_SUPPORT_DTLS
-    if (port->ssl_option.protocols & SW_SSL_DTLS) {
+    if (port->is_dtls()) {
         dtls::Buffer *buffer = (dtls::Buffer *) sw_malloc(sizeof(*buffer) + SW_BUFFER_SIZE_UDP);
         buffer->length = event->socket->read(buffer->data, SW_BUFFER_SIZE_UDP);
         dtls::Session *session = port->dtls_sessions->find(event->fd)->second;
@@ -589,7 +585,7 @@ static int ReactorThread_onRead(Reactor *reactor, Event *event) {
     }
 #endif
 
-    conn->last_recv_time = swoole_microtime();
+    conn->last_recv_time = microtime();
 
     int retval = port->onRead(reactor, port, event);
     if (!conn->active) {
@@ -1084,7 +1080,7 @@ void Server::start_heartbeat_thread() {
         SwooleTG.id = reactor_num;
 
         while (running) {
-            double checktime = swoole_microtime() - heartbeat_idle_time;
+            double checktime = microtime() - heartbeat_idle_time;
             foreach_connection([this, checktime](Connection *conn) {
                 if (conn->protect || conn->last_recv_time == 0 || conn->last_recv_time > checktime) {
                     return;
