@@ -45,7 +45,7 @@ void Server::worker_signal_init(void) {
     SwooleG.use_signalfd = SwooleG.enable_signalfd;
 
     swSignal_set(SIGHUP, nullptr);
-    swSignal_set(SIGPIPE, nullptr);
+    swSignal_set(SIGPIPE, SIG_IGN);
     swSignal_set(SIGUSR1, nullptr);
     swSignal_set(SIGUSR2, nullptr);
     // swSignal_set(SIGINT, Server::worker_signal_handler);
@@ -254,7 +254,7 @@ int Server::accept_task(EventData *task) {
             conn->ssl_client_cert = nullptr;
         }
 #endif
-        factory->end(task->info.fd);
+        factory->end(task->info.fd, false);
         break;
     }
     case SW_SERVER_EVENT_CONNECT: {
@@ -318,10 +318,6 @@ void Server::worker_start_callback() {
         SwooleG.process_type = SW_PROCESS_WORKER;
     }
 
-    if (enable_coroutine) {
-        SwooleG.enable_coroutine = 1;
-    }
-
     int is_root = !geteuid();
     struct passwd *_passwd = nullptr;
     struct group *_group = nullptr;
@@ -597,9 +593,9 @@ int Server::start_event_worker(Worker *worker) {
 }
 
 /**
- * Send data to ReactorThread
+ * [Worker/TaskWorker/Master] Send data to ReactorThread
  */
-ssize_t Server::send_to_reactor_thread(EventData *ev_data, size_t sendn, SessionId session_id) {
+ssize_t Server::send_to_reactor_thread(const EventData *ev_data, size_t sendn, SessionId session_id) {
     Socket *pipe_sock = get_reactor_thread_pipe(session_id, ev_data->info.reactor_id);
     if (SwooleTG.reactor) {
         return SwooleTG.reactor->write(SwooleTG.reactor, pipe_sock, ev_data, sendn);
@@ -608,6 +604,15 @@ ssize_t Server::send_to_reactor_thread(EventData *ev_data, size_t sendn, Session
     }
 }
 
+ssize_t Server::send_to_reactor_thread(const DataHead *head, const iovec *iov, size_t iovcnt, SessionId session_id) {
+    Socket *pipe_sock = get_reactor_thread_pipe(session_id, head->reactor_id);
+    if (SwooleTG.reactor) {
+        return swoole_event_writev(pipe_sock, iov, iovcnt);
+    } else {
+        return pipe_sock->writev_blocking(iov, iovcnt);
+    }
+}
+
 /**
  * send message from worker to another worker
  */
@@ -665,7 +670,7 @@ _read_from_pipe:
              */
             if (recv_chunk_count >= SW_WORKER_MAX_RECV_CHUNK_COUNT) {
                 swTraceLog(SW_TRACE_WORKER,
-                           "worker process[%lu] receives the chunk data to the maximum[%d], return to event loop",
+                           "worker process[%u] receives the chunk data to the maximum[%d], return to event loop",
                            SwooleG.process_id,
                            recv_chunk_count);
                 return SW_OK;
