@@ -104,6 +104,10 @@ ssize_t swoole_event_write(Socket *socket, const void *data, size_t len) {
     return SwooleTG.reactor->write(SwooleTG.reactor, socket, data, len);
 }
 
+ssize_t swoole_event_writev(swoole::network::Socket *socket, const iovec *iov, size_t iovcnt) {
+    return SwooleTG.reactor->writev(SwooleTG.reactor, socket, iov, iovcnt);
+}
+
 bool swoole_event_set_handler(int fdtype, ReactorHandler handler) {
     return SwooleTG.reactor->set_handler(fdtype, handler);
 }
