@@ -60,23 +60,27 @@ extern int php_get_gid_by_name(const char *name, gid_t *gid);
 #endif
 
 #if PHP_VERSION_ID < 70400
-static size_t php_stdiop_write(php_stream *stream, const char *buf, size_t count);
-static size_t php_stdiop_read(php_stream *stream, char *buf, size_t count);
+static size_t sw_php_stdiop_write(php_stream *stream, const char *buf, size_t count);
+static size_t sw_php_stdiop_read(php_stream *stream, char *buf, size_t count);
 #else
-static ssize_t php_stdiop_write(php_stream *stream, const char *buf, size_t count);
-static ssize_t php_stdiop_read(php_stream *stream, char *buf, size_t count);
+static ssize_t sw_php_stdiop_write(php_stream *stream, const char *buf, size_t count);
+static ssize_t sw_php_stdiop_read(php_stream *stream, char *buf, size_t count);
 #endif
 static int sw_php_stdiop_close(php_stream *stream, int close_handle);
-static int php_stdiop_stat(php_stream *stream, php_stream_statbuf *ssb);
-static int php_stdiop_flush(php_stream *stream);
-static int php_stdiop_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffset);
-static int php_stdiop_set_option(php_stream *stream, int option, int value, void *ptrparam);
-static int php_stdiop_cast(php_stream *stream, int castas, void **ret);
+static int sw_php_stdiop_stat(php_stream *stream, php_stream_statbuf *ssb);
+static int sw_php_stdiop_flush(php_stream *stream);
+static int sw_php_stdiop_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffset);
+static int sw_php_stdiop_set_option(php_stream *stream, int option, int value, void *ptrparam);
+static int sw_php_stdiop_cast(php_stream *stream, int castas, void **ret);
 
 static void php_stream_mode_sanitize_fdopen_fopencookie(php_stream *stream, char *result);
 static php_stream *_sw_php_stream_fopen_from_fd_int(int fd, const char *mode, const char *persistent_id STREAMS_DC);
 static php_stream *_sw_php_stream_fopen_from_fd(int fd, const char *mode, const char *persistent_id STREAMS_DC);
 
+static inline zend_bool file_can_poll(zend_stat_t *_stat) {
+    return S_ISCHR(_stat->st_mode) || S_ISSOCK(_stat->st_mode) || S_ISFIFO(_stat->st_mode);
+}
+
 static int sw_php_stream_parse_fopen_modes(const char *mode, int *open_flags)
 {
     int flags;
@@ -152,45 +156,47 @@ static int sw_php_stream_parse_fopen_modes(const char *mode, int *open_flags)
 /* {{{ ------- STDIO stream implementation -------*/
 
 typedef struct {
-	FILE *file;
-	int fd;					/* underlying file descriptor */
-	unsigned is_process_pipe:1;	/* use pclose instead of fclose */
-	unsigned is_pipe:1;			/* don't try and seek */
-	unsigned cached_fstat:1;	/* sb is valid */
-	unsigned is_pipe_blocking:1; /* allow blocking read() on pipes, currently Windows only */
-	unsigned _reserved:28;
+    FILE *file;
+    int fd;                 /* underlying file descriptor */
+    unsigned is_process_pipe:1; /* use pclose instead of fclose */
+    unsigned is_pipe:1;     /* stream is an actual pipe, currently Windows only*/
+    unsigned cached_fstat:1;    /* sb is valid */
+    unsigned is_pipe_blocking:1; /* allow blocking read() on pipes, currently Windows only */
+    unsigned no_forced_fstat:1;  /* Use fstat cache even if forced */
+    unsigned is_seekable:1;     /* don't try and seek, if not set */
+    unsigned _reserved:26;
 
-	int lock_flag;			/* stores the lock state */
-	zend_string *temp_name;	/* if non-null, this is the path to a temporary file that
-							 * is to be deleted when the stream is closed */
+    int lock_flag;          /* stores the lock state */
+    zend_string *temp_name; /* if non-null, this is the path to a temporary file that
+                             * is to be deleted when the stream is closed */
 #if HAVE_FLUSHIO
-	char last_op;
+    char last_op;
 #endif
 
 #if HAVE_MMAP
-	char *last_mapped_addr;
-	size_t last_mapped_len;
+    char *last_mapped_addr;
+    size_t last_mapped_len;
 #endif
 #ifdef PHP_WIN32
-	char *last_mapped_addr;
-	HANDLE file_mapping;
+    char *last_mapped_addr;
+    HANDLE file_mapping;
 #endif
 
-	zend_stat_t sb;
+    zend_stat_t sb;
 } php_stdio_stream_data;
 
 #define PHP_STDIOP_GET_FD(anfd, data)	anfd = (data)->file ? fileno((data)->file) : (data)->fd
 
 static php_stream_ops sw_php_stream_stdio_ops = {
-    php_stdiop_write,
-    php_stdiop_read,
+    sw_php_stdiop_write,
+    sw_php_stdiop_read,
     sw_php_stdiop_close,
-    php_stdiop_flush,
+    sw_php_stdiop_flush,
     "STDIO/coroutine",
-    php_stdiop_seek,
-    php_stdiop_cast,
-    php_stdiop_stat,
-    php_stdiop_set_option
+    sw_php_stdiop_seek,
+    sw_php_stdiop_cast,
+    sw_php_stdiop_stat,
+    sw_php_stdiop_set_option,
 };
 
 static int do_fstat(php_stdio_stream_data *d, int force)
@@ -222,38 +228,44 @@ static php_stream *_sw_php_stream_fopen_from_fd_int(int fd, const char *mode, co
 }
 
 #if PHP_VERSION_ID < 70400
-static size_t php_stdiop_write(php_stream *stream, const char *buf, size_t count)
+static size_t sw_php_stdiop_write(php_stream *stream, const char *buf, size_t count)
 #else
-static ssize_t php_stdiop_write(php_stream *stream, const char *buf, size_t count)
+static ssize_t sw_php_stdiop_write(php_stream *stream, const char *buf, size_t count)
 #endif
 {
     php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
 
     assert(data != NULL);
 
-    if (data->fd >= 0)
-    {
-        int bytes_written = write(data->fd, buf, count);
+    if (data->fd >= 0) {
+        if (file_can_poll(&data->sb)) {
+            if (!swoole_coroutine_socket_exists(data->fd) && swoole_coroutine_socket_create(data->fd) < 0) {
+                stream->eof = 1;
+                return -1;
+            }
+            return swoole_coroutine_write(data->fd, buf, count);
+        } else {
+            int bytes_written = write(data->fd, buf, count);
 #if PHP_VERSION_ID < 70400
-        if (bytes_written < 0)
-        {
-            return 0;
-        }
-        return (size_t) bytes_written;
+            if (bytes_written < 0)
+            {
+                return 0;
+            }
+            return (size_t) bytes_written;
 #else
-        return bytes_written;
+            return bytes_written;
 #endif
-    }
-    else
-    {
+        }
+
+    } else {
         return fwrite(buf, 1, count, data->file);
     }
 }
 
 #if PHP_VERSION_ID < 70400
-static size_t php_stdiop_read(php_stream *stream, char *buf, size_t count)
+static size_t sw_php_stdiop_read(php_stream *stream, char *buf, size_t count)
 #else
-static ssize_t php_stdiop_read(php_stream *stream, char *buf, size_t count)
+static ssize_t sw_php_stdiop_read(php_stream *stream, char *buf, size_t count)
 #endif
 {
     php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
@@ -263,17 +275,26 @@ static ssize_t php_stdiop_read(php_stream *stream, char *buf, size_t count)
 
     if (data->fd >= 0)
     {
-        ret = read(data->fd, buf, PLAIN_WRAP_BUF_SIZE(count));
-
-        if (ret == (size_t) -1 && errno == EINTR)
-        {
-            /* Read was interrupted, retry once,
-             If read still fails, giveup with feof==0
-             so script can retry if desired */
-            ret = read(data->fd, buf, PLAIN_WRAP_BUF_SIZE(count));
+        if (file_can_poll(&data->sb)) {
+            if (!swoole_coroutine_socket_exists(data->fd) && swoole_coroutine_socket_create(data->fd) < 0) {
+                stream->eof = 1;
+                return -1;
+            }
+            return swoole_coroutine_read(data->fd, buf, PLAIN_WRAP_BUF_SIZE(count));
         }
+        // file
+        else {
+            ret = read(data->fd, buf, PLAIN_WRAP_BUF_SIZE(count));
 
-        stream->eof = (ret == 0 || (ret == (size_t) -1 && errno != EWOULDBLOCK && errno != EINTR && errno != EBADF));
+            if (ret == (size_t) -1 && errno == EINTR) {
+                /* Read was interrupted, retry once,
+                 If read still fails, giveup with feof==0
+                 so script can retry if desired */
+                ret = read(data->fd, buf, PLAIN_WRAP_BUF_SIZE(count));
+            }
+            stream->eof =
+                    (ret == 0 || (ret == (size_t) -1 && errno != EWOULDBLOCK && errno != EINTR && errno != EBADF));
+        }
     }
     else
     {
@@ -351,7 +372,7 @@ static int sw_php_stdiop_close(php_stream *stream, int close_handle)
     return ret;
 }
 
-static int php_stdiop_flush(php_stream *stream)
+static int sw_php_stdiop_flush(php_stream *stream)
 {
     php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
 
@@ -369,7 +390,7 @@ static int php_stdiop_flush(php_stream *stream)
     return 0;
 }
 
-static int php_stdiop_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffset)
+static int sw_php_stdiop_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffset)
 {
     php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
     int ret;
@@ -402,7 +423,7 @@ static int php_stdiop_seek(php_stream *stream, zend_off_t offset, int whence, ze
     }
 }
 
-static int php_stdiop_cast(php_stream *stream, int castas, void **ret)
+static int sw_php_stdiop_cast(php_stream *stream, int castas, void **ret)
 {
     php_socket_t fd;
     php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
@@ -468,7 +489,7 @@ static int php_stdiop_cast(php_stream *stream, int castas, void **ret)
     }
 }
 
-static int php_stdiop_stat(php_stream *stream, php_stream_statbuf *ssb)
+static int sw_php_stdiop_stat(php_stream *stream, php_stream_statbuf *ssb)
 {
 	int ret;
 	php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
@@ -481,7 +502,7 @@ static int php_stdiop_stat(php_stream *stream, php_stream_statbuf *ssb)
 	return ret;
 }
 
-static int php_stdiop_set_option(php_stream *stream, int option, int value, void *ptrparam)
+static int sw_php_stdiop_set_option(php_stream *stream, int option, int value, void *ptrparam)
 {
 	php_stdio_stream_data *data = (php_stdio_stream_data*) stream->abstract;
 	size_t size;
@@ -938,39 +959,32 @@ static php_stream *stream_fopen_rel(const char *filename, const char *mode, zend
 	return NULL;
 }
 
-static php_stream *stream_opener(php_stream_wrapper *wrapper, const char *path, const char *mode, int options,
-        zend_string **opened_path, php_stream_context *context STREAMS_DC)
-{
-    if (((options & STREAM_DISABLE_OPEN_BASEDIR) == 0) && php_check_open_basedir(path))
-    {
+static php_stream* stream_opener(php_stream_wrapper *wrapper, const char *path, const char *mode, int options,
+                                 zend_string **opened_path, php_stream_context *context STREAMS_DC) {
+    if (((options & STREAM_DISABLE_OPEN_BASEDIR) == 0) && php_check_open_basedir(path)) {
         return NULL;
     }
 
+    php_stream *stream;
     /** phar_open_archive_fp, cannot use async-io */
-    if (
-        EG(current_execute_data) &&
-        EG(current_execute_data)->func &&
-        ZEND_USER_CODE(EG(current_execute_data)->func->type)
-    )
-    {
-        const zend_op* opline = EG(current_execute_data)->opline;
-        if (
-            opline && opline->opcode == ZEND_INCLUDE_OR_EVAL &&
-            (opline->extended_value & (ZEND_INCLUDE | ZEND_INCLUDE_ONCE | ZEND_REQUIRE | ZEND_REQUIRE_ONCE))
-        )
-        {
+    if (EG(current_execute_data) &&EG(current_execute_data)->func &&
+            ZEND_USER_CODE(EG(current_execute_data)->func->type)) {
+        const zend_op *opline = EG(current_execute_data)->opline;
+        if (opline && opline->opcode == ZEND_INCLUDE_OR_EVAL
+                && (opline->extended_value & (ZEND_INCLUDE | ZEND_INCLUDE_ONCE | ZEND_REQUIRE | ZEND_REQUIRE_ONCE))) {
             size_t path_len = strlen(path);
             size_t phar_len = sizeof(".phar") - 1;
-            if (path_len > phar_len && memcmp(path + path_len - phar_len, ".phar", phar_len) == 0)
-            {
-                return php_stream_fopen_rel(path, mode, opened_path, options);
+            if (path_len > phar_len && memcmp(path + path_len - phar_len, ".phar", phar_len) == 0) {
+                goto _open_for_include;
             }
         }
     }
     /** include file, cannot use async-io */
-    if (options & STREAM_OPEN_FOR_INCLUDE)
-    {
-        return php_stream_fopen_rel(path, mode, opened_path, options);
+    if (options & STREAM_OPEN_FOR_INCLUDE) {
+        _open_for_include:
+        stream = php_stream_fopen_rel(path, mode, opened_path, options);
+        stream->ops = php_swoole_get_ori_php_stream_stdio_ops();
+        return stream;
     }
 
     return stream_fopen_rel(path, mode, opened_path, options STREAMS_REL_CC);
